// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.utils.OperatingSystem;
import bio.dendogram.*;
import bio.dendogram.xml.*;
import bio.dendogram.GUI.*;
import java.awt.*;
import java.util.*;

public class ColoredTreeDrawing extends TreeDrawing {

  public ColoredTreeDrawing(ColoredTree root, Locale locale, OperatingSystem OS) {
    super(root, locale, OS);
    drawColors = true;
    coloringMethod = new ColoringMethod() {
      public Color getColor(XNode node) {
        return Color.black;
      }
    };
  }

  public Object clone() {
    ColoredTreeDrawing clone = new ColoredTreeDrawing((ColoredTree)getTree(), getLocale(), getOS());
    clone.setDrawColors(this.drawColors());
    clone.setColoringMethod(this.getColoringMethod());
    return clone;
  }

  /**
   * Set the elements to be plotted.
   */
  public void setDrawColors (boolean yesOrNot) { drawColors = yesOrNot; }

  public boolean drawColors() { return drawColors; }

  public void setColoringMethod(ColoringMethod method) { coloringMethod = method; }

  public ColoringMethod getColoringMethod() { return coloringMethod; }

  public Color chooseColor(Color[] colors) {
    Color result = Color.black;
    int numColors = 0;
    for(int i = 0; i < colors.length; i++) {
      if(!colors[i].equals(Color.black)) {
        numColors++; //couleur
        result = colors[i];
      }
    }
    if(numColors == 2) result = Color.magenta;
    if(numColors == 3) result = Color.orange;
    return result;
  }

  /**
   * Plotting methods (re-definition):
   */
  protected void plotLeafName(Graphics g, Node node, double x, double y) {
    setColor(g, node);
    super.plotLeafName(g, node, x, y);
  }
  protected void plotGroupName(Graphics g, Node node, double x, double y) {
    setColor(g, node);
    super.plotGroupName(g, node, x, y);
  }
  protected void plotLine(Graphics g, Node node, double x1, double y1, double x2, double y2) {
    setColor(g, node);
    super.plotLine(g, node, x1, y1, x2, y2);
  }
  protected void plotLengthValue(Graphics g, Node node, double x, double y) {
    setColor(g, node);
    super.plotLengthValue(g, node, x, y);
  }
  protected void plotBootstrapValue(Graphics g, Node node, double x, double y) {
    setColor(g, node);
    super.plotBootstrapValue(g, node, x, y);
  }
  protected void setColor(Graphics g, Node node) {
    if(drawColors) g.setColor(chooseColor(((ColoredNode)node).getColors()));
    else {
      g.setColor(coloringMethod.getColor((XNode)node));
    }
  }

  private boolean drawColors;
  private ColoringMethod coloringMethod;
  public static final Color COLOR_DEFAULT = Color.black;
}
