package TreeEditor;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import bio.dendogram.GUI.*;
import gnu.jtools.utils.gui.GraphicsTools;

/**
 * <p>Titre : Tree Editor</p>
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * <p>Socit : </p>
 * @author Julien Dutheil
 * @version 1.0
 */

public class PreferencesOptionDialog extends JDialog {

  private MainInterface mi;
  private JSpinner minimalLength;
  private JSpinner rightMargin;
  private JButton ok, cancel;

  public PreferencesOptionDialog(MainInterface mi) {
    super(mi, Baobab.message.getString("PreferencesOptionDialog.Title"), true);
    this.mi = mi;
    initGUI();
  }

  private void initGUI() {
    TreeDrawing td = mi.getTreeDrawing();
    minimalLength = new JSpinner(new SpinnerNumberModel(td.getMinimalLengthToDisplay(), 0., 100., 0.01));
    rightMargin   = new JSpinner(new SpinnerNumberModel(td.getMargins().right, 10, 2000, 10));
    ok = new JButton(Baobab.message.getString("ok"));
    ok.setMnemonic(Baobab.message.getString("okMnemo").charAt(0));
    ok.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Actualize all navigators:
        Navigator[] nav = mi.getNavigators();
        for(int i = 0; i < nav.length; i++) {
          actualizeTreeDrawing(nav[i].getTreeDrawing());
        }
        // Actualize main interface:
        actualizeTreeDrawing(mi.getTreeDrawing());

        setVisible(false);
      }
    });
    cancel = new JButton(Baobab.message.getString("cancel"));
    cancel.setMnemonic(Baobab.message.getString("cancelMnemo").charAt(0));
    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });

    // Layout:
    this.getContentPane().setLayout(new GridLayout(3,2));
    this.getContentPane().add(new JLabel(Baobab.message.getString("PreferencesOptionDialog.MinimalLength")));
    this.getContentPane().add(minimalLength);
    this.getContentPane().add(new JLabel(Baobab.message.getString("PreferencesOptionDialog.RightMargin")));
    this.getContentPane().add(rightMargin);
    this.getContentPane().add(ok);
    this.getContentPane().add(cancel);
    pack();
    GraphicsTools.center(this);
  }

  public double getMinimalLengthToDisplay() { return ((SpinnerNumberModel)minimalLength.getModel()).getNumber().doubleValue(); }

  public int getRightMargin() { return ((SpinnerNumberModel)rightMargin.getModel()).getNumber().intValue(); }

  public void actualizeTreeDrawing(TreeDrawing td) {
    td.setMinimalLengthToDisplay(getMinimalLengthToDisplay());
    Insets in = td.getMargins();
    in.right = getRightMargin();
    td.setMargins(in);
    td.scale();
    td.repaint();
  }
}
