// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

package bio.dendogram;

import java.util.ArrayList;
import java.util.Collection;

/**
 * <p>Title: Dendogram</p>
 * <p>Description: Set of Tools for dealing with phylogenetic trees (dendograms).</p>
 * <p>Copyright: 2002</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

public class NodeList extends ArrayList {

  public NodeList() {
  }

  public NodeList(int initialCapacity) {
    super(initialCapacity);
  }

  public NodeList(Collection c) {
    super(c);
  }

  public boolean add(Object o) {
    if(Node.class.isAssignableFrom(o.getClass())) return super.add(o);
    else return false;
  }

  public void add(int i, Object o) {
    if(Node.class.isAssignableFrom(o.getClass())) super.add(i, o);
  }

  public NodeList hardCopy() {
    NodeList hc = new NodeList();
    for(int i = 0; i < size(); i++) hc.add(((Node)get(i)).clone());
    return hc;
  }

  public TaxonomicGroup listOfLeavesNames() {
    TaxonomicGroup leavesNames = new TaxonomicGroup();
    for(int i = 0; i < size(); i++) {
      leavesNames.addAll( TreeTools.listOfLeavesNames( (Node)this.get(i)));
    }
    return(leavesNames);
  }
}
