package bio.dendogram.io;

/**
 * <p>Title: Dendogram</p>
 * <p>Description: Set of Tools for dealing with phylogenetic trees (dendograms).</p>
 * <p>Copyright: 2002</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

import gnu.jtools.utils.file.FileTools;
import gnu.jtools.utils.string.StringOperators;
import bio.dendogram.*;
import java.util.*;
import java.io.*;

public class NewickFormat
    extends AbstractOTree
    implements ITree {

  private static String removeComments(String chain) {
    //WARNING:ATTENTION Chain corresponds to a single tree (ie must end with ';').
    //Comments are within hooks before each tree: [].
    String result = new String(chain);
    for (; result.startsWith("[");
         result = result.substring(result.indexOf("]") + 1, result.length())) {
      ;
    }
    return result;
  }

  /**
   * Public method for reading trees from a file:
   */

  public Tree[] readTree(File inFile) throws IOException {
    String chain = "";
    ArrayList result = new ArrayList();
    BufferedReader input = new BufferedReader(new FileReader(inFile)); //the file to be read.
    String lineRead = input.readLine();
    //All the file is being read and lines are concatenated:
    do {
      chain += lineRead;
      lineRead = input.readLine();
    }
    while (lineRead != null);
    input.close(); //Closes the file. All data are stored into 'chain'.
    chain = StringOperators.removeChar(chain, ' '); //spaces are removed.
    StringTokenizer chains = new StringTokenizer(chain, ";");
    int nb = chains.countTokens();
    try {
      for (int i = 0; i < nb; i++) {
        String subChain = removeComments(chains.nextToken()); //comments are removed.
        //NB: semi-colon is removed by StringTokenizer.
        Tree tree = new Tree();
        tree.setRootNode(TreeTools.parenthesisToNode(tree, subChain));
        result.add(tree);
      }
    }
    catch (NumberFormatException e) {
      System.out.println(
          "Can't read length or bootstrap value: it is not a number.");
      throw new IOException();
    }
    return (Tree[]) (result.toArray(new Tree[0]));
  }

  /**
   * Writting trees into a file:
   */

  public void writeTree(Tree tree, PrintWriter pw) throws IOException {
    pw.println(TreeTools.treeToParenthesis(tree));
  }

  public static javax.swing.filechooser.FileFilter NEWICK_FILTER = new javax.
      swing.filechooser.FileFilter() {

    public boolean accept(File file) {
      if (file.isDirectory()) {
        return true;
      }
      String extension = FileTools.getExtension(file);
      if (extension != null) {
        if (extension.equals("newick") ||
            extension.equals("dnd") ||
            extension.equals("ph")) {
          return true;
        }
        else {
          return false;
        }
      }
      return false;
    }

    // The description of this filter
    public String getDescription() {
      return "Newick tree files (*.newick, *.dnd, *.ph)";
    }
  };

  /**
   * XML
   *
   *
   * tagDescription is a ReverseMap that contain:
   * - XTree attributes names as key
   * vs
   * - File attributes names as values.
   */

  /**
   * Reading methods:
   */


  /*  public static XTree elementToXTree(XMLElement elt, ReverseMap tagDescription) throws NullPointerException, NumberFormatException {
      if(elt == null) throw new NullPointerException();
      if(elt.getTagName().equals((String)tagDescription.getValue(TREE_BLOCK))) {//Tree block found.
        XTree tree = new XTree();

        //Name:
   XMLElement name = elt.removeChild((String)tagDescription.getValue(TREE_NAME));
        if(name != null) tree.setName(name.getElement());

        //Branches:
   XMLElement[] sons = elt.removeChildren((String)tagDescription.getValue(NODE));
        for(int i = 0; i < sons.length; i++) tree.addBranch(elementToNode(sons[i], tagDescription));

        //Other attributes if they exist:
        XMLElement elt2;
        while(elt.hasChild()) {
          elt2 = elt.removeChild(0);
          String tagName = (String)tagDescription.getKey(elt2.getTagName());
          if(tagName != null) {
            boolean editable = true;
            try { editable = Boolean.getBoolean((String)elt2.getAttributes().get("editable")); }
            catch(Exception e) {};
            if(tagName.equals(TREE_NAME)
               || tagName.equals(XTree.NODE_LENGTH)
               || tagName.equals(XTree.NODE_BOOTSTRAP)
               || tagName.equals(XSequence.SEQUENCE_CHAIN)
               ) editable = false;
            tree.setAttribute(tagName, elt2.getElement(), editable, editable);
          }
        }
        return tree;
      } else return null; //Not a tree element.
    }

    public static XTree elementToNode(XMLElement elt, ReverseMap tagDescription) throws NullPointerException {
      if(elt == null) throw new NullPointerException();
      if(elt.getTagName().equals((String)tagDescription.getValue(NODE))) {//Tree block found.
        XTree tree = new XTree();

   /*      //Name:
    XMLElement name = elt.removeChild((String)tagDescription.get("Node name"));
         if(name != null) tree.setName(name.getElement());

         //length
         XMLElement length = elt.removeChild((String)tagDescription.get("Branch length"));
    if(length != null) tree.setLength(Double.parseDouble(length.getElement()));

         //bootstrap
         XMLElement bootstrap = elt.removeChild((String)tagDescription.get("Bootstrap value"));
         if(bootstrap != null) tree.setBootstrap(Double.parseDouble(bootstrap.getElement()));
    */
   //Branches:
   /*      XMLElement[] sons = elt.removeChildren((String)tagDescription.getValue(NODE));
         for(int i = 0; i < sons.length; i++) tree.addBranch(elementToNode(sons[i], tagDescription));

         //Other attributes if they exist:
         XMLElement elt2;
         while(elt.hasChild()) {
           elt2 = elt.removeChild(0);
           String tagName = (String)tagDescription.getKey(elt2.getTagName());
           if(tagName != null) {
             if(tagName.equals(TREE_NAME)) tagName = XTree.NODE_NAME;//TREE_NAME and NODE_NAME may have the same tagName :-(
             boolean editable = true;
             try { editable = Boolean.getBoolean((String)elt2.getAttributes().get("editable")); }
             catch(Exception e) {};
             if(tagName.equals(XTree.NODE_NAME)
                || tagName.equals(XTree.NODE_LENGTH)
                || tagName.equals(XTree.NODE_BOOTSTRAP)
                || tagName.equals(XSequence.SEQUENCE_CHAIN)
                ) editable = false;
             tree.setAttribute(tagName, elt2.getElement(), editable, editable);
           }
         }
         return tree;
       } else return null; //Not a tree element.
     }

     public static XTree[] readXML(File inFile, ReverseMap tagDescription) throws IOException {
       //System.out.println("Reading XML tree file.");
       XMLDocument doc = XMLTools.readFile(inFile);
       XMLElement[] trees = doc.getChildren((String)tagDescription.getValue(TREE_BLOCK));
       List l = new ArrayList();
       try {
         for(int i = 0; i < trees.length; i++) {
           //System.out.println("Reading tree n°" + i);
           l.add(elementToXTree(trees[i], tagDescription));
         }
       } catch(NullPointerException e) {
         e.printStackTrace();
         throw new IOException();
       }
       return (XTree[])l.toArray(new XTree[0]);
     }

    */




   /**
    * Writing methods:
    */
   /*
    public static XMLElement toXMLElement(XTree tree, ReverseMap tagDescription) {
       //Create element:
       XMLElement elt = new XMLElement((String)tagDescription.get("Node"));

    /*    //Add node name:
        if(tree.hasName()) elt.addChild(new XMLElement((String)tagDescription.get("Node name"), tree.getName()));

        //Add node length:
        if(tree.hasLength()) elt.addChild(new XMLElement((String)tagDescription.get("Branch length"), "" + tree.getLength()));

        //Add node bootstrap:
        if(tree.hasBootstrap()) elt.addChild(new XMLElement((String)tagDescription.get("Bootstrap value"), "" + tree.getBootstrap()));
     */
    //Add attributes for this node:
    /*    Attribute[] attributes = (Attribute[])tree.getAttributes().entrySet().toArray(new Attribute[0]);
        for(int i = 0; i < attributes.length; i++) {
          Attribute attribute = attributes[i];
     String tagName = (String)tagDescription.getValue(attribute.getName());
          if(tagName != null && attribute.getValue() != null) {
            HashMap hm = new HashMap(1);
            hm.put("editable", new Boolean(attributes[i].isNameEditable()));
            elt.addChild( new XMLElement( tagName
                                          , hm
                                          , attribute.getValue().toString()
                                          )
                                        );
          }
        }

        //Add son nodes:
        for(int i = 0; i < tree.numberOfSons(); i++) {
          elt.addChild(toXMLElement((XTree)tree.branchAt(i), tagDescription));
        }

        //Return result:
        return elt;
      }

      public static void writeXML(XTree tree, File output, String title, ReverseMap tagDescription) throws IOException {
        XMLDocument doc = new XMLDocument((String)tagDescription.get(TREES_ROOT_NAME), null);
        XMLElement elt = toXMLElement(tree, tagDescription);
                   elt.setTagName((String)tagDescription.get(TREE_BLOCK));
                   elt.addChild(new XMLElement((String)tagDescription.get(TREE_NAME), tree.getName()));
        doc.addChild(elt);
        XMLTools.writeDocument(doc, output);
      }
      public static void writeXML(XTree[] trees, File output, String title, ReverseMap tagDescription) throws IOException {
        XMLDocument doc = new XMLDocument((String)tagDescription.getValue(TREES_ROOT_NAME), null);
        for(int i = 0; i < trees.length; i++) {
          XTree tree = trees[i];
     XMLElement elt = new XMLElement((String)tagDescription.get(TREE_BLOCK));
                     elt.addChild(new XMLElement((String)tagDescription.get(TREE_NAME), "Tree " + i));
                     elt.addChild(toXMLElement(tree, tagDescription));
          doc.addChild(elt);
        }
        XMLTools.writeDocument(doc, output);
      }
      public static void writeXML(List trees, File output, String title, ReverseMap tagDescription) throws IOException {
        XMLDocument doc = new XMLDocument((String)tagDescription.getValue(TREES_ROOT_NAME), null);
        int count = 1;
        for(Iterator i = trees.iterator(); i.hasNext();) {
          XTree tree = (XTree)i.next();
     XMLElement elt = new XMLElement((String)tagDescription.getValue(TREE_BLOCK));
     elt.addChild(new XMLElement((String)tagDescription.getValue(TREE_NAME)
     , tree.getName() == null ? ("Tree " + count) : tree.getName()
                         )
                     );
                     elt.addChild(toXMLElement(tree, tagDescription));
          doc.addChild(elt);
          count++;
        }
        XMLTools.writeDocument(doc, output);
      }*/

}
