
/**
 * Titre : Tree Editor<p>
 * @author Julien Dutheil
 */
package bio.dendogram.print;

import gnu.jtools.postscript.PSGraphics;
import bio.dendogram.GUI.*;
import java.awt.print.*;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.*;

public class TreePaginater extends Book {

  public TreePaginater(TreeDrawing treeDrawing, int numberMaxOfLeavesInAPage, PageFormat pageFormat) {
    this.treeDrawing = (TreeDrawing)treeDrawing.clone();
    int numberOfLeaves = treeDrawing.getTree().getNumberOfLeaves();
    int numberOfPages = new Double(Math.ceil(((double)((numberOfLeaves)))/((double)(numberMaxOfLeavesInAPage)))).intValue();
    int widthOfPrintableArea = widthOfOnePage;
    int heightOfPrintableArea = new Double(heightOfOnePage / numberOfPages).intValue();
    this.treeDrawing.setPreferredSize(widthOfOnePage, heightOfOnePage);
    this.treeDrawing.setDrawBranchesNumbers(false);
    this.treeDrawing.setDrawClickableNodes(false);
    Rectangle printableArea;
    for(int i = 0; i < numberOfPages; i++) {
      //Definition of the printable area:
      printableArea = new Rectangle(0, i * heightOfPrintableArea, widthOfPrintableArea, heightOfPrintableArea);
      //Creating a new page:
      TreePage newPage = new TreePage(this.treeDrawing, printableArea);
      this.append(newPage, pageFormat);
    }
  }

  public void printAsPostScript(File file) throws IOException {
    PSGraphics psGraphics = new PSGraphics();
    for(int i = 0; i < this.getNumberOfPages(); i++) {
      System.out.println("Printing page " + i);
      try { this.getPrintable(i).print(psGraphics, this.getPageFormat(i), 1); }
      catch(PrinterException ex) {
        ex.printStackTrace();
      }
      //Writing a new page:
      psGraphics.newPage();
    }
    psGraphics.write(file);
  }

  private TreeDrawing treeDrawing;
  private int widthOfOnePage = 500;
  private int heightOfOnePage = 700;
}

class TreePage implements Printable {

  public TreePage(TreeDrawing treeDrawing, Rectangle printableArea) {
    this.printableArea = printableArea;
    this.treeDrawing = treeDrawing;
  }

  public PageFormat getPageFormat() { return pageFormat; }

  public int print (Graphics g, PageFormat pageFormat, int pageIndex) throws PrinterException {
    this.pageFormat = pageFormat;
    if(pageIndex > 1) return NO_SUCH_PAGE;
    else {//page drawn:
      g.translate(50, 60);
      treeDrawing.paintPartOfComponent(g, printableArea);
      return PAGE_EXISTS;
    }
  }

  private TreeDrawing treeDrawing;
  private PageFormat pageFormat;
  private Rectangle printableArea;
}
