package bio.dendogram.statistics;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import bio.dendogram.*;
import bio.dendogram.xml.*;
import gnu.jtools.stats.array.ArrayStatTools;
import gnu.jtools.utils.attributes.AttributeNotFoundException;

public class PIC {

  public static void ancestor(XNode node, String variable) throws AttributeNotFoundException {
    if(!node.isLeaf()) {
      double[] x = new double[node.getNumberOfSons()];
      for(int i = 0; i < node.getNumberOfSons(); i++) {
        XNode son = (XNode)node.getSon(i);
        ancestor(son, variable);
        if(!son.hasAttribute(variable)) throw new AttributeNotFoundException("Attribute \"" + variable + "\" not found in node " + node.getId() + ".");
        x[i] = Double.parseDouble(son.getAttribute(variable).toString());
      }
      node.setAttribute(variable, new Double(ArrayStatTools.mean(x)), true, true);
    }
  }

  public static void ancestorWeighted(XNode node, String variable) throws AttributeNotFoundException, NodeException {
    if(node.isLeaf()) {
      if(!node.hasLength()) throw new NodeException(node, "No branch length.");
      double var = node.getLengthValue();
      node.setAttribute(variable + ".variance", new Double(var), true, true);
    } else {
      double[] l = new  double[node.getNumberOfSons()];
      for(int i = 0; i < node.getNumberOfSons(); i++) {
        XNode son = (XNode)node.getSon(i);
        ancestorWeighted(son, variable);
        if(!son.hasLength()) throw new NodeException(son, "No branch length.");
        l[i] = son.getLengthValue();
      }
      if(node.hasFather()) {
        // This is not the root ;-)
        if(!node.hasLength()) throw new NodeException(node, "No branch length.");
        double var = node.getLengthValue() +
            ArrayStatTools.prod(l) / ArrayStatTools.sum(l);
        node.setAttribute(variable + ".variance", new Double(var), true, true);
      }
      double[] x = new double[node.getNumberOfSons()];
      for(int i = 0; i < node.getNumberOfSons(); i++) {
        XNode son = (XNode)node.getSon(i);
        if(!((XNode)node.getSon(i)).hasAttribute(variable)) throw new AttributeNotFoundException(variable);
        x[i] = Double.parseDouble(son.getAttribute(variable).toString());
      }
      double numerator = 0;
      double denominator = 0;
      for(int i = 0; i < node.getNumberOfSons(); i++) {
        numerator += (1/l[i]) * x[i];
        denominator += 1/l[i];
      }
      node.setAttribute(variable, new Double(numerator/denominator), true, true);
    }
  }

  public static void pic(XNode node, String variable) {
    if(!node.isLeaf()) {
      // For now suppose tree is bifurcating.
      double x1 = Double.parseDouble(((XNode)node.getSon(0)).getAttribute(variable).toString());
      double x2 = Double.parseDouble(((XNode)node.getSon(1)).getAttribute(variable).toString());
      node.setAttribute(variable + ".contrast", new Double(x2 - x1), true, true);
    }
  }
}
