// -------------------------------------------------------------------------
//
// Copyright or © or Copr. CNRS, (November 16, 2004)
//
// This software is a library providing tool for bioinformatics.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

package bio.sequences;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * @author Julien Dutheil
 * @version 1.0
 */

import gnu.jtools.utils.string.TextTools;

/**
 * @brief A low level implementation of the Alphabet interface.
 *
 * This class implements common method to all alphabets.
 * It define a sletter structure used to store information for each state and a
 * contains a vector of this structure. All methods are based uppon this vector,
 * but does not provide any method to initialized it. This must be done by each
 * constructor of the derived classes.
 *
 * @see Alphabet
 */
public abstract class AbstractAlphabet implements Alphabet {

  // sletter structure : an alphabet is a vector of sletter
  protected class sletter {
    /**
     * @brief Base or acid n° (i.e. -1 for gap (-))
     */
    public int num;

    /**
     * @brief The char of base or amino-acid. May be more than one char (codons)!
     */
    public String letter;

    /**
     * @brief Abbreviation for amino-acids (not used in nucleic alphabets).
     */
    public String abbr;

    /**
     * @brief Full name of base or acid (i.e. Adenine)
     */
    public String name;

    public boolean equals(sletter s) { return name == s.name; }

  }

  /**
   * @brief Alphabet : vector of struct sletter (defined in Alphabet.h).
   */
  protected sletter[] alphabet;

  /**
   * @name Implement these methods from the Alphabet interface.
   *
   * @{
   */
  public int getNumberOfChars() { return alphabet.length; }

  public String getName(String letter) throws BadCharException {
    for (int i = 0; i < alphabet.length; i++)
      if (alphabet[i].letter.equalsIgnoreCase(letter)) return alphabet[i].name;

    throw new BadCharException(letter, "AbstractAlphabet::getName : Specified base unknown", this);
  }

  public String getName(int letter) throws BadIntException {
    for (int i = 0 ; i < alphabet.length ; i++)
        if (alphabet[i].num == letter) return alphabet[i].name;

    throw new BadIntException(letter, "AbstractAlphabet::getName : Specified base unknown", this);
  }

  public int charToInt(String letter) throws BadCharException {
    for (int i = 0; i < alphabet.length; i++)
        if (alphabet[i].letter.equalsIgnoreCase(letter)) return alphabet[i].num;

    throw new BadCharException(letter, "AbstractAlphabet::charToInt : Specified base unknown", this);
  }

  public String intToChar(int letter) throws BadIntException {
    for (int i = 0; i < alphabet.length; i++)
        if (alphabet[i].num == letter) return alphabet[i].letter;

    throw new BadIntException(letter, "AbstractAlphabet::intToChar : Specified base unknown", this);
  }

  public boolean isIntInAlphabet(int letter) {
    for (int i = 0; i < alphabet.length; i++)
        if (alphabet[i].num == letter) return true;
        return false;
  }

  public boolean isCharInAlphabet(String letter) {
    for (int i = 0; i < alphabet.length; i++)
        if (alphabet[i].letter.equalsIgnoreCase(letter)) return true;
        return false;
  }
  /** @} */

}
