package bio.sequences;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import java.util.*;
import bio.*;

/**
 * @brief Low level implementation of the OrderedSequenceContainer class.
 *
 * This abstract class provides an alphabet and comments, with associated methods.
 * It also implements many manipulation on sequences methods.
 */
public abstract class AbstractSequenceContainer
    implements OrderedSequenceContainer {

  /**
   * @brief The container's alphabet.
   */
  protected Alphabet _alphabet;

  /**
   * @brief The container's comments.
   */
  protected ArrayList _comments;

  /**
   * @brief This constructor initialize the alphabet pointer.
   *
   * This constructor is to be called by constructors of derived classes.
   *
   * @param alpha The alphabet to be associated to this container.
   */
  public AbstractSequenceContainer(Alphabet alpha) {
    _alphabet = alpha;
    _comments = new ArrayList();
  }

  /**
   * @name From the SequenceContainer interface
   *
   * @{
   */
  public Alphabet getAlphabet() {  return _alphabet;  }

  public Vint getContent(String name) throws SequenceNotFoundException { return getSequenceRef(name).getContent(); }

  public String toString(String name) throws SequenceNotFoundException { return getSequenceRef(name).toString(); }

  public List getComments(String name) throws SequenceNotFoundException { return getSequenceRef(name).getComments(); }

  public void setComments(String name, List comments) throws SequenceNotFoundException {
    int pos = getSequencePosition(name);
    setComments(pos, comments);
  }

  public List getGeneralComments() { return new ArrayList(_comments); }

  public void setGeneralComments(List comments) { _comments = new ArrayList(comments); }

  public void deleteGeneralComments() { _comments.clear(); }

  /** @} */

  /**
   * @name From the OrderedSequenceContainer interface
   *
   * @{
   */
  public String getName(int i) throws IndexOutOfBoundsException { return getSequenceRef(i).getName(); }
  public Vint getContent(int i) throws IndexOutOfBoundsException { return getSequenceRef(i).getContent(); }
  public String toString(int i) throws IndexOutOfBoundsException { return getSequenceRef(i).toString(); }
  public List getComments(int i) throws IndexOutOfBoundsException { return getSequenceRef(i).getComments(); }
  /** @} */

  /**
   * @name Inner methods.
   *
   * @{
   */
  public abstract Sequence getSequenceRef(String name) throws SequenceNotFoundException;
  public abstract Sequence getSequenceRef(int i) throws IndexOutOfBoundsException;
  /** @} */

}
