package bio.sequences;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

/**
 * @brief The alphabet exception base class.
 * @see Exception
 */
class AlphabetException extends Exception {

        protected Alphabet alphabet;

        /**
         * @brief Build a new AlphabetException.
         * @param text A message to be passed to the exception hierarchy.
         * @param alpha A const pointer toward the alphabet that threw the exception.
         */
        AlphabetException(String text, Alphabet alpha) {
          super("AlphabetException: " + text + (alpha != null ? "(" + (alpha.getAlphabetType()) + ")" : ""));
          alphabet = alpha;
        }
        AlphabetException(String text) {
          super("AlphabetException: " + text);
          alphabet = null;
        }

        /**
         * @brief Get the alphabet that threw the exception.
         * @return a const pointer toward the alphabet.
         */
        Alphabet getAlphabet() { return alphabet; }
};
