package bio.sequences;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

public class AlphabetTools {

  /**
   * @brief Character identification method for sequence's alphabet identification
   *
   * (used in method getAlphabetFromSequence() implemented in class Sequence)
   * Return :
   * -1 if character can't be nucleic or proteic
   *  0 ------------------ identify alphabet type (letters A, C, D, G, H, K, M, N, R, S, V, W, X, Y, -)
   *  1 ------------ identify nucleic alphabet    (letters B, O, ?, 0)
   *  2 ------------ may identify DNA alphabet    (letter T)
   *  3 ------------ identify RNA alphabet        (letter U)
   *  4 --------------------- proteic alphabet    (letters E, F, I, L, P, Q)
   *
   * @param acid The character to test.
   * @return The type code.
   */
  public static int getType(char acid) {
    // up specified character to capital letter
    acid = Character.toUpperCase(acid);

    if (acid=='A' || acid=='C' || acid=='D' || acid=='G' || acid=='H' || acid=='K'
        || acid=='M' || acid=='N' || acid=='R' || acid=='S' || acid=='V'
        || acid=='W' || acid=='X' || acid=='Y' || acid=='-') return 0;

    if ( acid=='B' || acid=='O' || acid=='?' || acid=='0') return 1;

    if (acid=='T') return 2;

    if (acid=='U') return 3;

    if (acid=='E' || acid=='F' || acid=='I' || acid=='L' || acid=='P' || acid=='Q') return 4;

    return -1;
  }

  /**
   * @brief This checks that all characters in the alphabet are coded by a string of same length.
   *
   * This method is used when states are coded by more than one character, typically: codon alphabets.
   *
   * @param alphabet The alphabet to check.
   * @return True if all text description have the same length (e.g. 3 for codon alphabet).
   */
  public static boolean checkAlphabetCodingSize(Alphabet alphabet) throws AlphabetException {
    if(alphabet.getNumberOfChars() == 0) return true; //Will this really happen?
    int size = alphabet.intToChar(0).length();
    for(int i = 1; i < alphabet.getNumberOfTypes(); i++) {
      if(alphabet.intToChar(i).length() != size) return false;
    }
    return true;
  }

  /**
   * @brief In case that all states in the given alphabet have a string description of same length,
   * send this length; e.g. 3 for codon alphabets.
   *
   * @param alphabet The alphabet to analyse.
   * @return The common size of all text descriptionif there is one. Else throws an AlphabetException.
   */
  public static int getAlphabetCodingSize(Alphabet alphabet) throws AlphabetException {
    if(!checkAlphabetCodingSize(alphabet)) throw new AlphabetException("Bad alphabet in function Alphabet::getAlphabetCodingSize().");
    return alphabet.intToChar(0).length();
  }

}
