package bio.sequences.GUI;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import bio.sequences.*;

public class SequenceEditorPane extends JPanel {

  private int windowSiteBegin, windowSiteEnd;
  private int windowSeqBegin, windowSeqEnd;
  private int gridWidth, gridHeight;
  private StringBuffer[] seqBuff;
  private String[] seqNames;
  private SequenceNamesEditorPane namesEditor;
  private Map colorMap;

  public class SequenceNamesEditorPane extends JPanel {

    public SequenceNamesEditorPane() {
      super.setBackground(Color.LIGHT_GRAY);
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);
      int y = 1;
      for(int i = windowSeqBegin; i < Math.min(windowSeqEnd, seqBuff.length); i++) {
        g.drawString(seqNames[i], 0, y * gridHeight );
        y++;
      }
    }
  }

  //----------------------------------------------------------------------------

  public SequenceEditorPane() {
    super.setBackground(Color.LIGHT_GRAY);
    colorMap = PROTEIN_COLORS;
    windowSiteBegin = 0;
    windowSeqBegin = 0;
    setWindowSize(100, 50);
    gridWidth = 10;
    gridHeight = 15;
    super.setPreferredSize(new Dimension((windowSiteEnd - windowSiteBegin) * gridWidth, (windowSeqEnd - windowSeqBegin) * gridHeight));
    namesEditor = new SequenceNamesEditorPane();
  }

  public void setSiteWindowPositions(int begin, int end) {
    windowSiteBegin = Math.min(begin, end);
    windowSiteEnd   = Math.max(begin, end);
    setPreferredSize(new Dimension((windowSiteEnd - windowSiteBegin) * gridWidth, (windowSeqEnd - windowSeqBegin) * gridHeight));
    repaint();
  }

  public void setSequenceWindowPositions(int begin, int end) {
    windowSeqBegin = Math.min(begin, end);
    windowSeqEnd   = Math.max(begin, end);
    setPreferredSize(new Dimension((windowSiteEnd - windowSiteBegin) * gridWidth, (windowSeqEnd - windowSeqBegin) * gridHeight));
    repaint();
    namesEditor.repaint();
  }

  public Dimension getWindowSize() { return new Dimension(windowSiteEnd - windowSiteBegin, windowSeqEnd - windowSeqBegin); }

  public void setWindowSize(int width, int height) {
    windowSiteEnd = windowSiteBegin + width;
    windowSeqEnd  = windowSeqBegin  + height;
  }

  public Dimension getGridSize() { return new Dimension(gridWidth, gridHeight); }

  public void setGridSize(int width, int height) {
    gridWidth = width;
    gridHeight = height;
    setPreferredSize(new Dimension((windowSiteEnd - windowSiteBegin) * gridWidth, (windowSeqEnd - windowSeqBegin) * gridHeight));
  }

  public void loadSequences(SequenceContainer sequences) {
    seqNames = sequences.getSequencesNames();
    seqBuff = new StringBuffer[seqNames.length];
    for(int i = 0; i < seqNames.length; i++) {
      try {
        seqBuff[i] = new StringBuffer(sequences.getSequence(seqNames[i]).
                                      toString());
      }
      catch (SequenceNotFoundException snfe) {
        //Can't happen!
        snfe.printStackTrace();
      }
    }
  }

  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    int y = 1;
    for(int i = windowSeqBegin; i < Math.min(windowSeqEnd, seqBuff.length); i++) {
      StringBuffer sb = seqBuff[i];
      int x = 0;
      for(int j = windowSiteBegin; j < Math.min(windowSiteEnd, sb.length()); j++) {
        char c = sb.charAt(j);
        Color col = (Color)colorMap.get(new Character(c));
        if(col == null) g.setColor(Color.BLACK);
        else g.setColor(col);
        g.drawString("" + c, x * gridWidth, y * gridHeight);
        x++;
      }
      y++;
    }
  }

  public JPanel getNamesPanel() { return namesEditor; }

  public static final Map PROTEIN_COLORS = new HashMap();
  static {
      PROTEIN_COLORS.put(new Character('A'), Color.YELLOW);
      PROTEIN_COLORS.put(new Character('R'), Color.RED);
      PROTEIN_COLORS.put(new Character('N'), Color.RED);
      PROTEIN_COLORS.put(new Character('D'), Color.RED);
      PROTEIN_COLORS.put(new Character('C'), Color.CYAN);
      PROTEIN_COLORS.put(new Character('Q'), Color.RED);
      PROTEIN_COLORS.put(new Character('E'), Color.RED);
      PROTEIN_COLORS.put(new Character('G'), Color.YELLOW);
      PROTEIN_COLORS.put(new Character('H'), Color.RED);
      PROTEIN_COLORS.put(new Character('I'), Color.GREEN);
      PROTEIN_COLORS.put(new Character('L'), Color.GREEN);
      PROTEIN_COLORS.put(new Character('K'), Color.RED);
      PROTEIN_COLORS.put(new Character('M'), Color.GREEN);
      PROTEIN_COLORS.put(new Character('F'), Color.BLUE);
      PROTEIN_COLORS.put(new Character('P'), Color.YELLOW);
      PROTEIN_COLORS.put(new Character('S'), Color.YELLOW);
      PROTEIN_COLORS.put(new Character('T'), Color.YELLOW);
      PROTEIN_COLORS.put(new Character('W'), Color.CYAN);
      PROTEIN_COLORS.put(new Character('Y'), Color.BLUE);
      PROTEIN_COLORS.put(new Character('V'), Color.GREEN);
      PROTEIN_COLORS.put(new Character('-'), Color.WHITE);

  }
  public static final Map NUCLEIC_ACIDS_COLORS = new HashMap();
  static {
    NUCLEIC_ACIDS_COLORS.put(new Character('A'), Color.RED);
    NUCLEIC_ACIDS_COLORS.put(new Character('T'), Color.BLUE);
    NUCLEIC_ACIDS_COLORS.put(new Character('G'), Color.YELLOW);
    NUCLEIC_ACIDS_COLORS.put(new Character('C'), Color.GREEN);
    NUCLEIC_ACIDS_COLORS.put(new Character('U'), Color.BLUE);
  }
}

