package bio.sequences.io;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */
import bio.sequences.*;
import gnu.jtools.utils.file.FileTools;
import gnu.jtools.utils.string.TextTools;
import java.io.*;
import java.util.*;
import javax.swing.filechooser.*;

public class Clustal
    extends AbstractIOSequence {

  /**
   * @brief Build a new Fasta object.
   *
   * @param charsByLine Number of character per line when writing files.
   */
  public Clustal(int charsByLine) {
    super(charsByLine);
  }

  public Clustal() {
    super();
  }

  public void read(File file, VectorSequenceContainer sc) throws Exception {
    //!!! ClustalX add 6 spaces at longest name.

    BufferedReader br = new BufferedReader(new FileReader(file));

    Alphabet alpha = sc.getAlphabet();
    List sequences = new ArrayList();

    String lineRead = "";

    List comments = new ArrayList(1);
    comments.add(FileTools.getNextLine(br)); // First line gives file generator.

    lineRead = FileTools.getNextLine(br); // This is the first sequence of the first block.

    int beginSeq = lineRead.lastIndexOf("      ") + 6;
    if (beginSeq == -1) {
      throw new IOException("Clustal::read. Bad intput file.");
    }

    int countSequences = 0;

    //Read first sequences block:
    boolean test = true;
    do {
      sequences.add(new Sequence(lineRead.substring(0, beginSeq - 6),
                                 lineRead.substring(beginSeq), alpha));
      countSequences++;
      lineRead = br.readLine();
      test = lineRead != null && !TextTools.isVoid(lineRead) && !TextTools.isVoid(lineRead.substring(0, beginSeq - 6));
    } while (test);

    // Read other blocks
    lineRead = FileTools.getNextLine(br); // Read first sequence of next block.
    while (lineRead != null) {
      // Read next block:
      for (int i = 0; i < countSequences; i++) { // Complete sequences
        if (TextTools.isVoid(lineRead)) {
          throw new IOException("Clustal::read. Bad intput file.");
        }
        ((Sequence)sequences.get(i)).append(lineRead.substring(beginSeq));
        lineRead = br.readLine();
      }
      // Here lineRead is the first line after the end of the block.
      lineRead = FileTools.getNextLine(br);
      // Drop consensus line if it exists:
      //if (lineRead != null && TextTools.isVoid(lineRead.substring(0, beginSeq - 6))) {
      //  lineRead = FileTools.getNextLine(br);
      //}
    }

    for (int i = 0; i < countSequences; i++) {
      sc.addSequence((Sequence)sequences.get(i), true);
    }
    sc.setGeneralComments(comments);
  }

  public static javax.swing.filechooser.FileFilter CLUSTALX_FILTER = new javax.
      swing.filechooser.FileFilter() {
    public boolean accept(java.io.File file) {
      if (file.isDirectory()) {
        return true;
      }
      String extension = FileTools.getExtension(file);
      if (extension != null) {
        if (extension.equals("clustal") ||
            extension.equals("aln")) {
          return true;
        }
        else {
          return false;
        }
      }
      return false;
    }

    // The description of this filter
    public String getDescription() {
      return "Clustal files (*.clustal, *.aln)";
    }
  };

  public String getFormatName() {
    return "Clustal";
  }

  public String getFormatDescription() {
    return "The Clustal alignment tool output format.";
  }

}
