package bio.sequences.io;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import bio.sequences.*;
import gnu.jtools.utils.file.FileTools;
import gnu.jtools.utils.string.TextTools;
import java.io.*;
import java.util.*;
import javax.swing.filechooser.*;

public class Mase
    extends AbstractIOSequence {

  /**
   * @brief Build a new Mase object.
   *
   * @param charsByLine Number of character per line when writing files.
   */
  public Mase(int charsByLine) {
    super(charsByLine);
  }

  public Mase() {
    super();
  }

  public static javax.swing.filechooser.FileFilter MASE_FILTER = new javax.
      swing.filechooser.FileFilter() {

    public boolean accept(java.io.File file) {
      if (file.isDirectory()) {
        return true;
      }
      String extension = FileTools.getExtension(file);
      if (extension != null) {
        if (extension.equals("mase") ||
            extension.equals("mas")) {
          return true;
        }
        else {
          return false;
        }
      }
      return false;
    }

    // The description of this filter
    public String getDescription() {
      return "Mase files (*.mase, *.mas)";
    }
  };

  public void read(File file, VectorSequenceContainer vsc) throws Exception {
    // Checking the existence of specified file
    BufferedReader br = new BufferedReader(new FileReader(file));

    // Initialization
    List seqComments = new ArrayList(), fileComments = new ArrayList();
    String temp = null, name = "", sequence = "";
    boolean comments = false;

    // Get current general comments is VectorSequenceContainer
    fileComments = vsc.getGeneralComments();

    // Main loop : for all file lines
    temp = br.readLine();
    while (temp != null) {
      // If first character is ;
      if (temp.charAt(0) == ';') {
        // If second character is also ;
        if (temp.charAt(1) == ';') {
          // File commentaries isolation
          temp = temp.substring(2); // Characters ;; deletion
          if (!TextTools.isVoid(temp)) {
            fileComments.add(temp);
          }
        } else {
          // If a name and a sequence were founded
          if ( (!name.equals("")) && (!sequence.equals(""))) {
            // New sequence creation, and addition in existing VectorSequenceContainer
            Sequence seq = new Sequence(name, sequence, seqComments,
                                        vsc.getAlphabet());
            vsc.addSequence(seq, true);
            name = "";
            sequence = "";
            seqComments.clear();
          }

          // Sequence commentaries isolation
          temp = temp.substring(1); // Character ; deletion
          if (!TextTools.isVoid(temp)) {
            seqComments.add(temp);
          }
          comments = true;
        }
      }
      else {
        // If sequence commentaries were just isolated
        if (comments) {
          // Sequence name isolation
          name = temp;
          comments = false;
        }
        else {
          sequence += temp; // Sequence isolation
        }
      }
      temp = br.readLine();
    }

    // Addition of the last sequence in file
    if ( (name != "") && (sequence != "")) {
      Sequence seq = new Sequence(name, sequence, seqComments, vsc.getAlphabet());
      vsc.addSequence(seq, true);
    }

    // Set new general comments in VectorSequenceContainer (old + new comments)
    vsc.setGeneralComments(fileComments);

    br.close();
  }

  public void write(File file, SequenceContainer sc, boolean overwrite) throws
      Exception {
    // Open file in specified mode
    PrintWriter pw = new PrintWriter(new FileWriter(file, overwrite));

    List comments = sc.getGeneralComments();

    // Writing all general commentaries in file
    for (int i = 0; i < comments.size(); i++) {
      pw.println(";;" + comments.get(i));
    }

    String seq = null, temp = null; // Initialization

    // Main loop : for all sequences
    String[] names = sc.getSequencesNames();
    for (int i = 0; i < names.length; i++) {
      comments = sc.getComments(names[i]);

      // Writing all sequence commentaries in file
      // If no comments are associated with current sequence, an empy commentary line will be writed
      if (comments == null || comments.size() == 0) {
        pw.println(";");
      }
      else {
        for (int j = 0; j < comments.size(); j++) {
          pw.println(";" + comments.get(j));
        }
      }

      // Sequence name writing
      pw.println(names[i]);

      // Sequence cutting to specified characters number per line
      seq = sc.toString(names[i]);
      while (!TextTools.isVoid(seq)) {
        if (seq.length() > _charsByLine) {
          temp = new String(seq.substring(0, _charsByLine));
          pw.println(temp);
          seq = seq.substring(_charsByLine);
        }
        else {
          pw.println(seq);
          seq = "";
        }
      }
    }

    pw.close();
  }

  /**
   * @name The IOSequence interface.
   *
   * @{
   */
  public String getFormatName() {
    return "MASE file";
  }

  public String getFormatDescription() {
    return "Optional file comments (preceeded by ;;), sequence comments (preceeded by ;), sequence name, sequence";
  }

  /** @} */

}
