package bio.sequences.io;

import gnu.jtools.utils.file.FileTools;
import gnu.jtools.utils.string.TextTools;
import java.io.*;
import java.util.*;

public class NEXUSTools {

  private static String removeFirstBlanks(String s) {
    StringBuffer sb = new StringBuffer(s);
    while(sb.charAt(0)==' ' || sb.charAt(0)=='\t' || sb.charAt(0)=='\n') {
      sb.deleteCharAt(0);
    }
    return sb.toString();
  }

  public static String[] getLines(File inFile) throws IOException {
    BufferedReader  input = new BufferedReader (new FileReader(inFile)); //the file to be read.
    String lineRead = input.readLine();
    String result = "";
    do {
      result += lineRead;
      lineRead = input.readLine();
    } while(lineRead != null);
    StringTokenizer analyse = new StringTokenizer(result, ";", false);
    int n = analyse.countTokens();
    String[] lines = new String[n];
    for(int i = 0; i < n; i++) lines[i] = removeFirstBlanks(analyse.nextToken());
    input.close();
    return lines;
  }

  public static NEXUSCommand getCommand(String line) {
    line = TextTools.replace(line, "\t", " ");
    int index = line.indexOf(" ");
    if(index < 0) return null;
    String commandName = line.substring(0, index);
    String parameters = line.substring(index + 1);
    NEXUSCommand command = new NEXUSCommand(commandName);
    StringTokenizer analyse1, analyse2;
    int n1, n2;
    String delimiter1 = ",", delimiter2 = "="; //Default values
    //Particular cases:
    if(commandName.toUpperCase().equals("TRANSLATE")) delimiter2 = " ";
    if(commandName.toUpperCase().equals("TREE")) delimiter1 = "";
    analyse1 = new StringTokenizer(parameters, delimiter1);
    n1 = analyse1.countTokens();
    for(int i = 0; i < n1; i++) {
      String parameter = analyse1.nextToken();
      analyse2 = new StringTokenizer(parameter, delimiter2);
      n2 = analyse2.countTokens();
      if(n2 != 2) continue;
        String s1 = analyse2.nextToken(), s2 = analyse2.nextToken();
        //if(commandName.toUpperCase().equals("TRANSLATE")) {
        //  command.put(s2, s1);
        //} else
        command.put(s1, s2);
    }
    return command;
  }

  public static NEXUSEnvironment[] getEnvironments(String[] lines) {
    NEXUSEnvironment currentEnvironment = null;
    ArrayList environments = new ArrayList();
    int iB, iE;

    for(int i = 0; i < lines.length; i++) {
      iB = lines[i].toUpperCase().indexOf("BEGIN");
      if(iB > -1) {//Begin new environment
        currentEnvironment = new NEXUSEnvironment(lines[i].substring(iB + 6));
      } else {
        String blank = TextTools.removeBlanks(lines[i]).toUpperCase();
        if(blank.equals("END")) {//End of currentEnvironment:
        environments.add(currentEnvironment);
        } else {//command line of the current environment:
          currentEnvironment.addCommand(getCommand(lines[i]));
        }
      }
    }

    return (NEXUSEnvironment[])environments.toArray(new NEXUSEnvironment[0]);
  }

  public static class NEXUSCommand extends HashMap{

    public NEXUSCommand(String name) {
      this.name = name;
    }

    public String getName() { return name; }

    public void setName(String name) { this.name = name; }

    private String name;
  }

  public static class NEXUSEnvironment {

    public NEXUSEnvironment(String name) {
      this.name = name;
      commands = new ArrayList();
    }

    public String getName() { return name; }

    public void setName(String name) { this.name = name; }

    public void addCommand(NEXUSCommand command) {
      commands.add(command);
    }

    public NEXUSCommand getCommand(int index) {
      return (NEXUSCommand)commands.get(index);
    }

    public int numberOfCommands() { return commands.size(); }

    private String name;
    private ArrayList commands;

  }
  public static FileFilter NEXUS_FILTER = new FileFilter() {

    public boolean accept(java.io.File file) {
      if(file.isDirectory()) return true;
      String extension=FileTools.getExtension(file);
      if(extension!=null) {
        if(extension.equals("nex") ||
           extension.equals("nx")) return true;
        else return false;
      }
      return false;
    }

    // The description of this filter
    public String getDescription() {
      return "Nexus files (*.nex, *.nx)";
    }
  };

}
