package bio.sequences.io;

import gnu.jtools.utils.file.format.FormatAndFileChooser;
import gnu.jtools.utils.xmltools.ParserOptionPanel;

import bio.sequences.*;
import bio.sequences.xml.*;
import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.util.*;

public class SequenceIODialog extends FormatAndFileChooser {

  protected Alphabet alphabet;
  protected Fasta fasta;
  protected Mase mase;
  protected Phylip phylip, phylipExtended, phylipInterleaved, phylipExtendedInterleaved;
  protected Clustal clustal;

  public SequenceIODialog(Frame owner, boolean readOrWrite, boolean encapsulateXmlTagPanel, Alphabet alphabet) {
    message = ResourceBundle.getBundle("bio.sequences.io.bundle", new Locale("en", "US"));
    parserPanel = new XMLParserPanel(readOrWrite);
    this.alphabet = alphabet;
    fasta = new Fasta();
    mase = new Mase();
    phylip = new Phylip(false, true);
    phylipExtended = new Phylip(true, true);
    phylipInterleaved = new Phylip(false, false);
    phylipExtendedInterleaved = new Phylip(true, false);
    clustal = new Clustal();
    //xmlIO    = new XMLTreeFormat();
    setReadOrWrite(readOrWrite);
    if(encapsulateXmlTagPanel) {
      parserDialog = new JDialog(owner, "XML Parser panel", true);
      parserDialog.getContentPane().setLayout(new BorderLayout());
      parserDialog.getContentPane().add(new JScrollPane(parserPanel), BorderLayout.CENTER);
      JToolBar toolBar = new JToolBar("Control");
      toolBar.add(new AbstractAction("Ok") {
        public void actionPerformed(ActionEvent e) { parserDialog.setVisible(false); }
      });
      parserDialog.getContentPane().add(toolBar, BorderLayout.SOUTH);
      parserDialog.pack();
    }
  }

  public void setReadOrWrite(boolean readOrWrite) {
    this.readOrWrite = readOrWrite;
    parserPanel.setReadOrWrite(readOrWrite);
//    if(readOrWrite) {
//      super.addActionListener(new ActionListener() {
//        public void actionPerformed(ActionEvent e) {
//          if (getSelectedFormat().equals("XML")) {
//            try {
//              parserPanel.setAvailableTags(xmlIO.getAvailableTags(getSelectedFile()));
//            } catch (IOException ioe) {
//              // Do nothing, file currently selected is not valid.
//            }
//          }
//        }
//      });
//    }
  }

  public SequenceContainer readSequenceFile() throws IOException {
    String format = super.getSelectedFormat();
    File input = getSelectedFile();
    try {
      if (format.equals("Mase")) return mase.read(input, alphabet);
      if (format.equals("Fasta")) return fasta.read(input, alphabet);
      if (format.equals("Phylip")) return phylip.read(input, alphabet);
      if (format.equals("Phylip (extended)")) return phylipExtended.read(input, alphabet);
      if (format.equals("Phylip (interleaved)")) return phylipInterleaved.read(input, alphabet);
      if (format.equals("Phylip (extended, interleaved)")) return phylipExtendedInterleaved.read(input, alphabet);
      if (format.equals("Clustal")) return clustal.read(input, alphabet);
      //if(format.equals("XML"     )) {
      //  xml.setTagDescription(parserPanel.getTagDescription());
      //  xml.read(input, alphabet);
      //}
      //if(format.equals("EMBL"    )) {
      //  if(EMBLOD == null) EMBLOD = new EMBLTools.EMBLOptionsDialog(this.getLocale()); //Create dialog box.
      //  EMBLOD.setVisible(true);
      //  Map tags =  EMBLOD.getTags();
      //  currentSelectedSequenceFile = EMBLTools.readFileInEMBL(input, tags);
      //}
    } catch(Exception e) {
      e.printStackTrace();
      throw new IOException("Error while opening sequence file.");
    }
    return null;
  }

  public void writeSequenceFile(SequenceContainer seqFile) throws IOException {
    File output = getSelectedFile();
    String format = super.getSelectedFormat();
    writeSequenceFile(seqFile, output, format);
  }

  public void writeSequenceFile(SequenceContainer seqFile, File output) throws IOException {
    String format = super.getSelectedFormat();
    writeSequenceFile(seqFile, output, format);
  }

  public void writeSequenceFile(SequenceContainer seqFile, String format) throws IOException {
    File output = getSelectedFile();
    writeSequenceFile(seqFile, output, format);
  }

  public void writeSequenceFile(SequenceContainer seqFile, File outFile, String format) throws IOException {
    int qts = super.showExistingFileDialog(outFile);
    if(qts == CANCEL) return;
    boolean overwrite = (qts != APPEND);
    try {
      if (format.equals("Mase")) mase.write(outFile, seqFile, overwrite);
      if (format.equals("Phylip")) phylip.write(outFile, seqFile, overwrite);
      if (format.equals("Phylip (extended)")) phylipExtended.write(outFile,
          seqFile, overwrite);
      if (format.equals("Phylip (interleaved)")) phylipInterleaved.write(
          outFile, seqFile, overwrite);
      if (format.equals("Phylip (extended, interleaved)"))
          phylipExtendedInterleaved.write(outFile, seqFile, overwrite);
      //if(format.equals("ClustalX"))  WriteSequenceFile.writeFileInClustalX(seqFile, output);
      if (format.equals("Fasta")) fasta.write(outFile, seqFile, overwrite);
      //if(format.equals("XML"     ))  WriteSequenceFile.writeFileInXML((XSequenceFile)seqFile, output, parserPanel.getTagDescription());
    } catch(SequenceNotAlignedException snae) {
      JOptionPane.showMessageDialog(this, message.getString("Error.SequencesNotAligned"), message.getString("ErrorTitle"), JOptionPane.ERROR_MESSAGE);
    } catch(Exception e) {
      e.printStackTrace();
      throw new IOException("Error while writing file " + outFile.getAbsolutePath());
    }
  }

  public int showOpenDialog(Component c) {
    setFormatsAvailable(READABLE_FORMATS_AVAILABLE, READABLE_FILTERS_AVAILABLE);
    return super.showOpenDialog(c);
  }

  public int showSaveDialog(Component c) {
    setFormatsAvailable(WRITABLE_FORMATS_AVAILABLE, WRITABLE_FILTERS_AVAILABLE);
    return super.showSaveDialog(c);
  }

  public ParserOptionPanel getParserOptionPanel() { return parserPanel; }

  private XMLParserPanel parserPanel;
  private JDialog parserDialog;
  private boolean readOrWrite;
  //private EMBLTools.EMBLOptionsDialog EMBLOD;
  private ResourceBundle message;
  public static final String[] WRITABLE_FORMATS_AVAILABLE =
  { "Mase",
    "Phylip",
    "Phylip (extended)",
    "Phylip (interleaved)",
    "Phylip (extended, interleaved)",
    //"ClustalX",
    "Fasta",
    //"XML"
  };
  public static final String[] READABLE_FORMATS_AVAILABLE =
  { "Mase",
    "Phylip",
    "Phylip (extended)",
    "Phylip (interleaved)",
    "Phylip (extended, interleaved)",
    "Clustal",
    "Fasta",
    //"XML",
    //"EMBL"
  };
  public static final FileFilter[] WRITABLE_FILTERS_AVAILABLE = {
    Mase.MASE_FILTER,
    Phylip.PHYLIP_FILTER,
    Phylip.PHYLIP_FILTER,
    Phylip.PHYLIP_FILTER,
    Phylip.PHYLIP_FILTER,
//    Clustal.CLUSTALX_FILTER,
    Fasta.FASTA_FILTER,
//    XMLTools.XML_FILTER
  };
  public static final FileFilter[] READABLE_FILTERS_AVAILABLE = {
    Mase.MASE_FILTER,
    Phylip.PHYLIP_FILTER,
    Phylip.PHYLIP_FILTER,
    Phylip.PHYLIP_FILTER,
    Phylip.PHYLIP_FILTER,
    Clustal.CLUSTALX_FILTER,
    Fasta.FASTA_FILTER,
//    XMLTools.XML_FILTER,
//    EMBLTools.EMBL_FILTER
  };
}
