/*
 * License package: tools for programs with license.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package gnu.jtools.license;

/**
 * <p>Title: License</p>
 * <p>Description: Tools for programs with license</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import gnu.jtools.utils.gui.GraphicsTools;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class LicenseDisplayDialog extends JDialog {

  public LicenseDisplayDialog(Dialog owner, String title, boolean modal, URL licenseURL) {
    super(owner, title, modal);
    licenseTermsURL = licenseURL;
    initGUI();
  }

  public LicenseDisplayDialog(Frame owner, String title, boolean modal, URL licenseURL) {
    super(owner, title, modal);
    licenseTermsURL = licenseURL;
    initGUI();
  }

  public LicenseDisplayDialog(String title, boolean modal, URL licenseURL) {
    super();
    super.setTitle(title);
    super.setModal(modal);
    licenseTermsURL = licenseURL;
    initGUI();
  }

  private void initGUI() {
    licenseTerms = new JTextPane();
    licenseTerms.setEditable(false);
    try {
      licenseTerms.setPage(licenseTermsURL);
    } catch(IOException e) {
      System.out.println("Can't find license file.");
      //e.printStackTrace();
      System.exit(-1);
    }

    accept = new JRadioButton("Accept the terms of the license", false);
    decline = new JRadioButton("Decline the terms of the license", true);
    ButtonGroup bg = new ButtonGroup();
    bg.add(accept);
    bg.add(decline);
    ok = new AbstractAction("Ok") {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    };
    toolBar = new JToolBar();
    toolBar.add(ok);
    toolBar.setFloatable(false);

    //Setting up the GUI:
    this.getContentPane().setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 2;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 100;
    JScrollPane scp = new JScrollPane(licenseTerms);
    scp.setPreferredSize(new Dimension(600, 600));
    this.getContentPane().add(scp, gbc);
    Box box = Box.createVerticalBox();
    box.add(accept);
    box.add(decline);
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 75;
    gbc.weighty = 0;
    this.getContentPane().add(box, gbc);
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 25;
    gbc.weighty = 0;
    this.getContentPane().add(toolBar, gbc);
    pack();
    GraphicsTools.centerHorizontally(this);
    GraphicsTools.adjust(this, 30, 30);
  }

  public boolean userAcceptsTheTermsOfTheLicense() {
    return accept.isSelected();
  }

  private JTextPane licenseTerms;
  private JRadioButton accept, decline;
  private JToolBar toolBar;
  private Action ok;
  private URL licenseTermsURL;
}
