/*
 * StatsTools package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package gnu.jtools.stats.array;

import gnu.jtools.stats.DimensionException;
import java.util.*;

public class ArrayOperators {

  public static double[] add(double[] array1, double[] array2) throws DimensionException {
    if(array1.length!=array2.length) throw new DimensionException();
    double[] result=new double[array1.length];
    for(int i=0; i<array1.length; i++) {
      result[i]=array1[i]+array2[i];
    }
    return result;
  }
  public static int[] add(int[] array1, int[] array2) throws DimensionException {
    if(array1.length!=array2.length) throw new DimensionException();
    int[] result=new int[array1.length];
    for(int i=0; i<array1.length; i++) {
      result[i]=array1[i]+array2[i];
    }
    return result;
  }

  public static double[] scalarMultiply(double[] array, double lambda) {
    double[] result=new double[array.length];
    for(int i=0; i<array.length; i++) {
      result[i] = lambda * array[i];
    }
    return result;
  }
  public static int[] scalarMultiply(int[] array, int lambda) {
    int[] result=new int[array.length];
    for(int i=0; i<array.length; i++) {
      result[i] = lambda * array[i];
    }
    return result;
  }

  public static double[] neg(double[] array) {
    double[] result=new double[array.length];
    for(int i=0; i<array.length; i++) {
      result[i]=-array[i];
    }
    return result;
  }
  public static int[] neg(int[] array) {
    int[] result=new int[array.length];
    for(int i=0; i<array.length; i++) {
      result[i]=-array[i];
    }
    return result;
  }

  public static boolean inter(boolean[] t) {
    boolean result=true;
    for(int i=0; i<t.length; i++) {
      result=result && t[i];
    }
    return result;
  }

  public static boolean union(boolean[] t) {
    boolean result=false;
    for(int i=0; i<t.length; i++) {
      result=result || t[i];
    }
    return result;
  }

  public static int indexOf(Object[] array, Object object) {
    int result=-1;
    for(int i=0; i<array.length; i++) {
      if(array[i].equals(object)) result=i;//object with similar properties found.
    }
    return result;
  }

  public static int indexOfRef(Object[] array, Object object) {
    int result=-1;
    for(int i=0; i<array.length; i++) {
      if(array[i]==object) result=i;//object not found.
    }
    return result;
  }

  public static Object[] idElements(Object[] list1, Object[] list2) {
    Vector result=new Vector(10);
    for(int i=0; i<list1.length; i++) {
      for(int j=0; j<list2.length; j++) {
        if(list1[i].equals(list2[j]) && result.indexOf(list1[i])==-1) {
          result.add(list1[i]);
        }
      }
    }
    return ((Vector)(result.clone())).toArray();//Nouveau objets (ni ceux de list1, ni ceux de list2).
  }

  public static Object[] repeatedELements(Object[] array) {
    Vector list=new Vector();
    for(int i=0; i<array.length; i++) list.add(array[i]);
    Vector result=new Vector(10);
    for(int i=0; i<list.size(); i++) {
      if(list.indexOf(list.get(i))!=list.lastIndexOf(list.get(i))) result.add(list.get(i));
    }
    return result.toArray();
  }

  public static void swapElements(double[] array, int i, int j) {
    double swap = array[i];
    array[i] = array[j];
    array[j] = swap;
  }
  public static void swapElements(int[] array, int i, int j) {
    int swap = array[i];
    array[i] = array[j];
    array[j] = swap;
  }

  public static double[] castFromArrayOfInt(int [] x) {
    double[] y = new double[x.length];
    for(int i = 0; i < y.length; i++) y[i] = (double)x[i];
    return y;
  }

}