package gnu.jtools.stats.gui;

import gnu.jtools.stats.table.Table;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 * <p>Title: Stats</p>
 * <p>Description: Statistics tools</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

public class SpreadSheet extends JTable {

  protected Table table;
  protected SpreadSheetTableModel sstm;
  protected ColumnPropertiesDialog cpd;
  //protected final JTable jTable = this;

  public SpreadSheet(Table table, Window owner) {
    super(new SpreadSheetTableModel(table));
    this.table = table;
    initGUI(owner);
  }

  private void initGUI(final Window owner) {
    cpd = new ColumnPropertiesDialog(owner, this);
    //cpd.setLocationRelativeTo(owner);
    super.setColumnSelectionAllowed(false);
    MouseAdapter listMouseListener = new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
            TableColumnModel columnModel = getColumnModel();
            int viewColumn = columnModel.getColumnIndexAtX(e.getX());
            int column = convertColumnIndexToModel(viewColumn);
            if (e.getClickCount() == 1 && column > 0) {
              cpd.setColumnIndex(column - 1);
              //cpd.setLocation(e.getPoint());
              cpd.setLocation(owner.getX() + e.getX(), owner.getY() + e.getY());
              cpd.setVisible(true);
            }
        }
    };
    getTableHeader().addMouseListener(listMouseListener);
  }

  public static final String[] TYPES = {"Integer", "Double", "Boolean", "String" };
  public static final Class[] CLASSES = { Integer.class, Double.class, Boolean.class, String.class };
  class ColumnPropertiesDialog extends JWindow {

    protected int columnIndex;
    protected JTextField columnName;
    protected JComboBox typeSelector;
    protected JButton ok;
    protected SpreadSheet spreadSheet;

    public ColumnPropertiesDialog(Window owner, SpreadSheet ss) {
      //super("Column properties");
      super(owner);
      spreadSheet = ss;
      initGUI();
      columnIndex = -1;
    }

    private void initGUI() {
      columnName = new JTextField(15);
      typeSelector = new JComboBox(TYPES);
      this.getContentPane().setLayout(new GridBagLayout());
      GridBagConstraints c = new GridBagConstraints();
      c.gridx = 0; c.gridy = 0;
      this.getContentPane().add(new Label("Column name"), c);
      c.gridx = 1; c.gridy = 0;
      this.getContentPane().add(columnName, c);
      c.gridx = 0; c.gridy = 1;
      this.getContentPane().add(new Label("Data type"), c);
      c.gridx = 1; c.gridy = 1;
      this.getContentPane().add(typeSelector, c);
      ok = new JButton("ok");
      ok.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          String newName = columnName.getText();
          // Test if new name is valid:
          // @todo!!!
          if(!newName.equals(table.getColumnName(columnIndex))) {
            table.setColumnName(columnIndex, newName);
            // Update table display:
            ( (SpreadSheetTableModel) spreadSheet.getModel()).
                fireTableStructureChanged();
          }
          Class newType = CLASSES[typeSelector.getSelectedIndex()];
          if(!newType.equals(table.getColumnClass(columnIndex))) {
            try {
              table.coerceColumnType(columnIndex, newType);
            } catch(NumberFormatException nfe) {
              JOptionPane.showMessageDialog(spreadSheet, "Cast toward type '" + typeSelector.getSelectedItem() +"' failed.", "Error!!!", JOptionPane.ERROR_MESSAGE);
            }
          }
          setVisible(false);
        }
      });
      c.gridx = 0; c.gridy = 2; c.gridwidth = 2;
      this.getContentPane().add(ok, c);
      pack();
    }

    public void setColumnIndex(int index) {
      columnIndex = index;
      columnName.setText(table.getColumnName(index));
      if(Integer.class.isAssignableFrom(table.getColumnClass(index))) typeSelector.setSelectedIndex(0);
      else if(Double.class.isAssignableFrom(table.getColumnClass(index))) typeSelector.setSelectedIndex(1);
      else if(Boolean.class.isAssignableFrom(table.getColumnClass(index))) typeSelector.setSelectedIndex(2);
      else if(String.class.isAssignableFrom(table.getColumnClass(index))) typeSelector.setSelectedIndex(3);
      else {
        System.err.println("Unexpected type found !!!");
      }

    }
  }

}
