package gnu.jtools.utils.attributes;

/**
 * <p>Titre : Attributes</p>
 * <p>Description : Basic classes to deal with attributes</p>
 * <p>Copyright : Copyright (c) 2002</p>
 * @author non attribu�
 * @version 1.0
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import javax.swing.event.*;

public class AttributesEditor extends JPanel {

  public AttributesEditor() {
    atm = new AttributesTableModel(new AttributesMap());
    table = new JTable(atm);

    initActions();
    setLayout(new BorderLayout());

    JScrollPane scroll = new JScrollPane(table);
    add(scroll, BorderLayout.CENTER);

    editBar = new JToolBar(JToolBar.HORIZONTAL);
    editBar.add(add);
    editBar.add(remove);
    this.add(editBar, BorderLayout.SOUTH);
    listeners = new ArrayList();
  }

  private void initActions() {

    add = new AbstractAction("Add") {
      public void actionPerformed(ActionEvent e) {
        atm.addRow("New attribute", "value", true, true);
        table.updateUI();
      }
    };
    final AttributesEditor attEd = this;
    remove = new AbstractAction("Remove") {
      public void actionPerformed(ActionEvent e) {
        int[] rows = table.getSelectedRows();
        if(rows.length == 1) rows[0] = table.getSelectedRow();
        Arrays.sort(rows);//Very important!
        for(int i = rows.length - 1; i >= 0; i--) { //Must remove last first!
          if(atm.getAttribute(rows[i]).isValueEditable()) {
            atm.removeRow(rows[i]);
            table.updateUI();
          } else JOptionPane.showMessageDialog( attEd
              , "This attribute can't be removed: " + atm.getAttribute(rows[i]).getName()
              , "Error."
              , JOptionPane.ERROR_MESSAGE
              );
        }
      }
    };
  }

  public void addTableModelListener(TableModelListener listener) {
    listeners.add(listener);
    atm.addTableModelListener(listener);
  }

  public String getAttributeValue(String attributeName) {
    for(int i = 0; i < atm.getRowCount(); i++) if(((String)atm.getValueAt(i, 0)).equals(attributeName)) return (String)atm.getValueAt(i, 1);
    return null;
  }

  public String getAttributeName(int index) {
    return (String)atm.getValueAt(index, 0);
  }

  public String getAttributeValue(int index) {
    return (String)atm.getValueAt(index, 1);
  }

  public TableModel getAttributesTableModel() { return atm; }

  public void setAttributesTableModel(AttributesTableModel atm) {
    this.atm = atm;
    for(int i = 0; i < listeners.size(); i++) atm.addTableModelListener((TableModelListener)listeners.get(i));
    //System.out.println("Attributes table model changed");
    table.setModel(atm);
  }


  public static class AttributesTableModel extends AbstractTableModel {

    public AttributesTableModel(AttributesMap am) {
      data = new ArrayList(Arrays.asList((Attribute[])am.attributes.toArray(new Attribute[0])));
    }

    public int getRowCount() { return data.size(); }

    public int getColumnCount() { return 2; }

    public String getColumnName(int column) { return columnNames[column]; }

    public boolean isCellEditable(int row, int column) {
      switch(column) {
        case 0 : return getAttribute(row).isNameEditable();
        case 1 : return getAttribute(row).isValueEditable();
      }
      return false;
    }

    public Object getValueAt(int row, int column) {
      if(column == 0) return ((Attribute)data.get(row)).getKey();
      if(column == 1) return ((Attribute)data.get(row)).getValue();
      return null;
    }

    public void setValueAt(Object value, int row, int column) {
      if(!isCellEditable(row, column)) return;
      if(column == 0) ((Attribute)data.get(row)).setName((String)value);
      if(column == 1) ((Attribute)data.get(row)).setValue(value);
      fireTableChanged(new TableModelEvent(this));
    }

    public void addRow(String name, String value) {
      Attribute att = new Attribute(name, value);
      data.add(att);
      fireTableChanged(new TableModelEvent(this));
    }

    public void addRow(String name, String value, boolean isNameEditable, boolean isValueEditable) {
      Attribute att = new Attribute(name, value, isNameEditable, isValueEditable);
      data.add(att);
      fireTableChanged(new TableModelEvent(this));
    }

    public void removeRow(int row) {
      if(row > -1 && row < this.getRowCount()) data.remove(row);
      fireTableChanged(new TableModelEvent(this));
    }

    public Attribute getAttribute(int i) {
      return (Attribute)data.get(i);
    }

    public AttributesMap getAttributes() {
      AttributesMap result = new AttributesMap();
      for(int i = 0; i < getRowCount(); i++) {
        result.put(this.getAttribute(i));
      }
      return result;
    }

    private List data;
  }

  protected JTable table;
  protected AttributesTableModel atm;
  protected Action add, remove;
  protected JToolBar editBar;
  private static String[] columnNames = { "Attribute: ", "Value: " };
  private List listeners;
}
