package gnu.jtools.utils.attributes;

/**
 * <p>Titre : Attributes</p>
 * <p>Description : Basic classes to deal with attributes</p>
 * <p>Copyright : Copyright (c) 2002</p>
 * @author Julien Dutheil
 * @version 1.0
 */

import gnu.jtools.utils.file.FileTools;
import gnu.jtools.utils.string.TextTools;
import java.util.*;
import java.io.*;

public class AttributesTools {

  public static AttributesMap getAttributesMap(String[] args, String delimiter) {
    AttributesMap am = new AttributesMap();
    for(int i = 0; i < args.length; i++) {
      int limit = args[i].indexOf(delimiter);
      am.put(args[i].substring(0, limit), args[i].substring(limit + delimiter.length()));
    }
    return am;
  }

  public static AttributesMap getAttributesMap(List argv, String delimiter) {
    AttributesMap am = new AttributesMap();
    for(int i = 0; i < argv.size(); i++) {
      //Make a few corrections first:
      String arg = removeComments((String)argv.get(i), "#", "\n");//remove shell comments.
      if(TextTools.isVoid(arg)) continue;
      arg = removeComments(arg, "//", "\n");//remove C simple comments.
      arg = removeComments(arg, "/*", "*/");//remove C multiple comments.
      arg = TextTools.removeFirstAndLastBlanks(arg);
      if(arg == "") continue; //Skipping void line.
      int limit = arg.indexOf(delimiter, 0);
      if(limit == -1) {//Invalid parameter
        System.err.println("WARNING!!! Parameter '" + arg + "' has been ignored at line " + i + ".");
      } else {
        am.put(TextTools.removeLastBlanks(arg.substring(0, limit)), TextTools.removeFirstBlanks(arg.substring(limit + delimiter.length())));
      }
    }
    return am;
  }

  public static AttributesMap getAttributesMapFromFile(File file, String delimiter) throws IOException {
    List lines = FileTools.putFileIntoStringList(file);
    return getAttributesMap(lines, delimiter);
  }

  public static void actualizeAttributesMap(AttributesMap attMap, AttributesMap atts) {
    Set keys = atts.keySet();
    for(Iterator i = keys.iterator(); i.hasNext(); ) {
      Object k = i.next();
      Object v = attMap.get(k);
      attMap.put(k, v);
    }
  }

  public static String removeComments(String s, String begin, String end) {
    StringBuffer r = new StringBuffer(s);
    int last = 0;
    do {
      int first = r.indexOf(begin, last);
      if(first == -1) return r.toString(); //No shell comment.
      //else:
      last = r.indexOf(end, first);
      if(last == -1) {
        r.delete(first, r.length());
      } else {
        r.delete(first, last);
      }
    } while(last != -1);
    return r.toString();
  }

}
