/*
 * CommandLine package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * Titre : commandLine<p>
 * Description : Reading command lines.<p>
 * @author Julien Dutheil
 * @version 1.0
 */
package gnu.jtools.utils.commandline;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Console {

  public Console() {
    variables = new ListOfVariables();
    this.currentDirectory=new File(System.getProperty("user.dir"));
  }

  public static String getCommandLine() {
    //Saisie d'une ligne de commande au clavier.
    String ligneLue=null;
    System.out.print(">");
    try {
      InputStreamReader lecture = new InputStreamReader(System.in);
      BufferedReader entree = new BufferedReader(lecture);
      ligneLue=entree.readLine();
    } catch (IOException e) {
      System.out.println("SOS is down for an unknown reason.");
      System.exit(0);
    }
    return ligneLue;
  }

  public static CommandLine compoundCommandLine(String commandLine) throws SyntaxeException, NotAParameterException {
    int n=commandLine.indexOf(";");
    if(n > -1) commandLine=commandLine.substring(0, n);
    else throw new SyntaxeException();
    StringTokenizer ana1 = new StringTokenizer(commandLine," ");
    n=ana1.countTokens();
    String name=ana1.nextToken();//Le nom de la commande.
    CommandLine result = new CommandLine(name, n-1);
    for(int i=1; i<n; i++) {
      String block=ana1.nextToken();
      StringTokenizer ana2=new StringTokenizer(block, "=");
      if(ana2.countTokens()!=2) throw new NotAParameterException(ana2.nextToken());
      result.add(new Argument(ana2.nextToken(), ana2.nextToken()));
      }
    return result;
    }

  public void exit() {
    System.out.println("Bye bye...");
    System.exit(0);
    }

  public ListOfVariables getVariables() { return variables; }

  public File getCurrentDirectory() { return currentDirectory; }
  public void setCurrentDirectory(File currentDirectory) {
    this.currentDirectory=currentDirectory;
    }

  private File currentDirectory;
  private ListOfVariables variables;

/**
  public static void main(String[] args) throws Exception {
    try {
        UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) { }
    System.out.println();
    System.out.println("********** Welcome to S.O.S.: ************");
    System.out.println("*       Sequence Operating System        *");
    System.out.println("*     Current Version is 1.00 beta.      *");
    System.out.println("******************************************");
    System.out.println();
    Console console=new Console();
    do {
      try { Command.letsDo(console, console.compoundCommandLine(console.getCommandLine())); }
      catch(SyntaxeException e)          { System.out.println("Syntaxe error. Must be: "+e.getGoodSyntaxe()); }
      catch(NotAParameterException e)    { System.out.println("Bad parameter: "+e.getBadParameter()); }
      catch(ArgumentNotFoundException e) { System.out.println("Argument not found: "+e.getArgumentName()); }
      catch(CommandNotFoundException e)  { System.out.println("Command not found: "+e.getCommandName()); }
      catch(IOException e)               { System.out.println("File not found: "+e.getMessage()); }
      catch(Exception e)                 { System.out.println("Some trouble here..."); e.printStackTrace(); }
      }
    while(true);

    }
*/
  }
