/*
 * Database package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package gnu.jtools.utils.database;

import gnu.jtools.utils.gui.GraphicsTools;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;

public class ConnectionDialog extends JDialog {

  public ConnectionDialog(Frame owner) {
    super(owner, "Connection Manager", true);
    closeOption = CANCEL;
    initGUI();
    pack();
    GraphicsTools.center(this);
  }

  private void initGUI() {
    driver = new JComboBox(availableDrivers);
    driver.setEditable(false);

    domain = new JTextField("localhost");
    //domain.setEnabled(false);

    port = new JTextField("5432");
    //port.setEnabled(false);

    database = new JTextField();

    login = new JTextField();

    password = new JPasswordField();

    connect = new JButton("Connect");
    connect.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
        closeOption = CONNECT;
      }
    });

    cancel = new JButton("Cancel");
    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
        closeOption = CANCEL;
      }
    });

    //Setting layout manager:
    Container c = this.getContentPane();
    c.setLayout(new GridLayout(7, 2));
    c.add(new JLabel("Driver:"));
    c.add(driver);
    c.add(new JLabel("Domain:"));
    c.add(domain);
    c.add(new JLabel("Port:"));
    c.add(port);
    c.add(new JLabel("Database:"));
    c.add(database);
    c.add(new JLabel("Login:"));
    c.add(login);
    c.add(new JLabel("Password:"));
    c.add(password);
    c.add(connect);
    c.add(cancel);
  }

  public void setVisible(boolean b) {
    closeOption = CANCEL;
    super.setVisible(b);
  }

  public Connection getConnection() throws Exception {
    if(closeOption == CANCEL) return null;
    
    int i = driver.getSelectedIndex();
    if(i == 0) Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    if(i == 1) Class.forName("com.mysql.jdbc.Driver");
    if(i == 2) Class.forName("org.postgresql.Driver");
    if(i > 0) System.out.println("Loading driver " + availableDriversNames[i]);
    String url = "jdbc:" + availableDrivers[i] + "://" + domain.getText() + ":" + port.getText() + "/" + database.getText();
    System.out.println("Executing " + url.toString());
    Connection db = DriverManager.getConnection(url, login.getText(), new String(password.getPassword()));
    return db;
  }

  private JComboBox driver;
  private JTextField domain;
  private JTextField port;
  private JTextField database;
  private JTextField login;
  private JPasswordField password;
  private JButton connect, cancel;
  private int closeOption;
  public static final String[] availableDriversNames = { "JDBC-ODBC", "MySQL and ConnectorJ (must be installed)", "postgresql (must be installed)" };
  public static final String[] availableDrivers = { "jdbc", "mysql", "postgresql" };
  public static final int CANCEL  = -1;
  public static final int CONNECT = 0;
}
