/*
 * Display package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package gnu.jtools.utils.display;

import java.io.*;

public class Console implements Terminal {

  public Console() {
  }

  public void display(String info) {
    System.out.print(info);
  }

  public void displayError(String error) {
    System.out.println("(!) Error: " + error);
  }

  public String readString() {
    String result = null;
    try {
      InputStreamReader reader = new InputStreamReader(System.in);
      BufferedReader input = new BufferedReader(reader);
      result = input.readLine();
    } catch(IOException e) {
      displayError("I/O");
      System.exit(0);
    }
  return result;
  }

  public int readInt() {
    int result = 0;
    try {
      String lineRead = readString();
      result = Integer.parseInt(lineRead);
    } catch(NumberFormatException e ) {
      display("*** Not a number ***");
    }
    return result;
  }

  public boolean readBoolean() {
    String lineRead = readString();
    String[] other = { "N", "No", "Non" };
    boolean result = true;
    for(int i = 0; i < other.length; i++) {
      if(lineRead.equalsIgnoreCase(other[i])) { result = false; break; }
    }
    return result;
  }

  public File getFile() {
    return new File(readString());
  }

  public void exit(int status) {
    System.exit(status);
  }
}