package gnu.jtools.utils.file.format;

import javax.swing.*;
import javax.swing.filechooser.*;
import java.awt.event.*;
import java.awt.*;
/*
 * Format package. You should have received this file along with another program.
 * See the program files for description and full license terms.
 * Copyright (C) 2003  Julien Dutheil
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.io.File;

/**
 * @author Julien Dutheil
 */

public class FormatAndFileChooser extends JFileChooser {

  public FormatAndFileChooser() {
    initGUI();
  }

  public FormatAndFileChooser(File currentDirectory) {
    super(currentDirectory);
    initGUI();
  }

  public FormatAndFileChooser(String currentDirectoryPath) {
    super(currentDirectoryPath);
    initGUI();
  }

  public void setFormatsAvailable(String[] formats, FileFilter[] filters) throws IllegalArgumentException {
    if(formats.length != filters.length) throw new IllegalArgumentException();
    this.formats.clear();
    this.filters = filters;
    this.resetChoosableFileFilters();
    fileFormat.removeAllItems();
    for(int i = 0; i < formats.length; i++) {
      this.formats.add(formats[i]);
      fileFormat.addItem(formats[i]);
    }
    this.addChoosableFileFilter(filters[0]);
    fileFormat.setSelectedIndex(0);
  }

  public void setSelectedFormat(String format) throws IllegalArgumentException {
    fileFormat.setSelectedIndex(formats.indexOf(format));
    fileFormat.updateUI();
  }

  private void initGUI() {
    formats = new java.util.ArrayList();
    fileFormat = new JComboBox();
    fileFormat.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(fileFormat.getSelectedIndex() > -1) {
          try {
            resetChoosableFileFilters();
          } catch (ArrayIndexOutOfBoundsException aiobe) {
            // Well this is because of a MacOSX Bug with Java 1.4!
          }
          addChoosableFileFilter(filters[fileFormat.getSelectedIndex()]);
        }
      }
    });
    panel = new JPanel();
    panel.setSize(new Dimension(60, 30));
    panel.add(fileFormat);
    panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.black), "Format:"));
    super.setAccessory(panel);
  }

  public String getSelectedFormat() {
    return (String)fileFormat.getSelectedItem();
  }

  public int getSelectedFormatIndex() {
    return fileFormat.getSelectedIndex();
  }

  public JPanel getPanel() { return panel; }

/*  public void addFormatSelectionListener(ActionListener al) {
    fileFormat.addActionListener(al);
    addActionListener(al);
  }

  public void removeFormatSelectionListener(ActionListener al) {
    fileFormat.removeActionListener(al);
    removeActionListener(al);
  }*/

  /**
   * Check if a given file exists.
   * If it does not, send 0 (the program may write the file).
   * If it does, ask for advice: the user may
   * - cancel the action and the method send -1,
   * - overwrite the file and the method send 1,
   * - append the file and the method send 2.
   */

  public int showExistingFileDialog(File file) {
    if(!file.exists()) return OK;
    String[] options = { "Overwrite", "Append" };
    int qts = JOptionPane.showOptionDialog( this
                                          , "File already exists.\nWhat do you want to do?"
                                          , "Warning..."
                                          , JOptionPane.CANCEL_OPTION
                                          , JOptionPane.QUESTION_MESSAGE
                                          , null
                                          , options
                                          , null
                                          );
    if(qts == 0) return OVERWRITE;
    if(qts == 1) return APPEND;
    return CANCEL;
  }

  protected java.util.List formats;
  protected JComboBox fileFormat;
  protected FileFilter[] filters;
  protected JPanel panel;
  public static final int CANCEL = -1;
  public static final int OK = 0;
  public static final int OVERWRITE = 1;
  public static final int APPEND = 2;
}
