package gnu.jtools.utils.gui;

import java.awt.*;

public class GraphicsTools {

  /**
   * Put this component at the center of the screen, keeping its original dimensions.
   *
   * @param c The component to adjust.
   */
  public static void center(Component c) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    c.setLocation((screen.width - c.getSize().width) / 2, (screen.height - c.getSize().height) / 2);
  }

  /**
   * Put this component in the middle of the width of the screen.
   *
   * Size and vertical position are kept.
   *
   * @param c The component to adjust.
   */
  public static void centerHorizontally(Component c) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    c.setLocation((screen.width - c.getSize().width) / 2, c.getLocation().y);
  }

  /**
   * Put the component in the middle of the heigth of the screen.
   *
   * Size and horizontal position are kept.
   *
   * @param c the component to adjust.
   */
  public static void centerVertically(Component c) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    c.setLocation(c.getLocation().x, (screen.height - c.getSize().height) / 2);
  }

  /**
   * Set the dimensions of a component to screen size minus margin values.
   *
   * NB: The component is then centered.
   *
   * @param c       The component to adjust.
   * @param hoffset Horizontal margin value.
   * @param voffset Vertical margin value.
   */
  public static void fullScreen(Component c, int hoffset, int voffset) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    c.setSize(new Dimension(screen.width - 2 * hoffset, screen.height - 2 * voffset));
    center(c);
  }

  /**
   * Maximize the width of a component according to screen size and margin value.
   *
   * @param c       The component to adjust.
   * @param hoffset Horizontal margin value.
   */
  public static void fullWidth(Component c, int hoffset) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    c.setSize(new Dimension(screen.width - 2 * hoffset, c.getSize().height));
    centerHorizontally(c);
  }

  /**
   * Maximize the height of a component according to screen size and margin value.
   *
   * @param c       The component to adjust.
   * @param voffset Vertical margin value.
   */
  public static void fullHeight(Component c, int voffset) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    c.setSize(new Dimension(c.getSize().width, screen.height - 2 * voffset));
    centerVertically(c);
  }

  /**
   * Adjust the dimensions of a component to screen size minus margin values.
   *
   * NB: The component is then centered.
   *
   * @param c       The component to adjust.
   * @param hoffset Horizontal margin value.
   * @param voffset Vertical margin value.
   */
  public static void adjust(Component c, int hoffset, int voffset) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    int w = Math.min(screen.height - 2 * hoffset, c.getSize().width);
    int h = Math.min(screen.height - 2 * voffset, c.getSize().height);
    c.setSize(new Dimension(w, h));
    center(c);
  }

  /**
   * Adjust the width of a component according to screen size and margin value.
   *
   * @param c       The component to adjust.
   * @param hoffset Horizontal margin value.
   */
  public static void adjustWidth(Component c, int hoffset) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    int w = Math.min(screen.height - 2 * hoffset, c.getSize().width);
    c.setSize(new Dimension(w, c.getSize().height));
    centerHorizontally(c);
  }

  /**
   * Adjust the height of a component according to screen size and margin value.
   *
   * @param c       The component to adjust.
   * @param voffset Vertical margin value.
   */
  public static void adjustHeight(Component c, int voffset) {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    int h = Math.min(screen.height - 2 * voffset, c.getSize().height);
    c.setSize(new Dimension(c.getSize().width, h));
    centerVertically(c);
  }
}