package gnu.jtools.utils.storage;

import java.util.*;

public class SymetricMap extends AbstractReverseMap<Object, Object> {

  public SymetricMap() {
    entrySet = new HashSet();
  }
  public SymetricMap(Map map) {
    entrySet = new HashSet(map.entrySet());
  }

  public Set entrySet() { return entrySet; }

  public Object put(SymetricMap.Entry entry) {
    if(!entrySet.contains(entry)) {
      entrySet.add(entry);
      return null;
    } else return entry;
  }

  public Object put(Object o1, Object o2) {
    Map.Entry entry = new SymetricMap.Entry(o1, o2);
    if(!entrySet.contains(entry)) {
      entrySet.add(entry);
      return null;
    } else return entry;
  }

  public Object get(Object key) {
    //System.out.println("Key : " + key.toString());
    Iterator i = entrySet().iterator();
    if (key == null) {
      while (i.hasNext()) {
        Entry e = (Entry) i.next();
        if (e.getKey() == null) return e.getValue();
        if (e.getValue() == null) return e.getKey();
      }
    } else {
      while (i.hasNext()) {
	Entry e = (Entry) i.next();
        //System.out.println(e.getKey().toString() + "\t" + e.getValue().toString());
	if (key.equals(e.getKey())) return e.getValue();
	if (key.equals(e.getValue())) return e.getKey();
      }
    }
    return null;
  }

  public String toString() {
    String result = "";
    for(Iterator i = entrySet.iterator(); i.hasNext();) {
      Entry e = (Entry)i.next();
      result += "Key:\t" + e.getKey() + "\tValue:\t" + e.getValue() + "\n";
    }
    return result;
  }

  protected Set entrySet;

  public static class Entry implements Map.Entry {
    public Entry(Object o1, Object o2) {
      this.key = o1;
      this.value = o2;
    }

    public Object getKey() { return key; }
    public Object getValue() { return value; }

    public Object setKey(Object key) {
      Object old = this.key;
      this.key = key;
      return old;
    }

    public Object setValue(Object value) {
      Object old = this.value;
      this.value = value;
      return old;
    }

    public boolean equals(Object o) {
      try {
        Entry entry = (Entry)o;
        if(key.equals(entry.getKey()) && value.equals(entry.getValue())) return true;
        if(key.equals(entry.getValue()) && value.equals(entry.getKey())) return true;
      } catch(ClassCastException e) { return false; }
      return false;
    }

    protected Object key, value;
  }
}