package gnu.jtools.utils.xmltools;

/**
 * <p>Titre : XMLTools</p>
 * <p>Description : XML Tools</p>
 * <p>Copyright : Copyright (c) 2002</p>
 * @author Julien Dutheil
 * @version 1.0
 */

import java.util.*;
import java.io.*;

public class XMLElement {

  //Constructors:
  public XMLElement(String tagName) {
    this.tagName = tagName;
    this.attributes = null;
    this.element = null;
    this.parent = null;
    this.children = new ArrayList();
  }
  public XMLElement(String tagName, String element) {
    this.tagName = tagName;
    this.attributes = null;
    this.element = element;
    this.parent = null;
    this.children = new ArrayList();
  }
  public XMLElement(String tagName, HashMap attributes) {
    this.tagName = tagName;
    this.attributes = attributes;
    this.element = null;
    this.parent = null;
    this.children = new ArrayList();
  }
  public XMLElement(String tagName, HashMap attributes, String element) {
    this.tagName = tagName;
    this.attributes = attributes;
    this.element = element;
    this.parent = null;
    this.children = new ArrayList();
  }
  public XMLElement(String tagName, HashMap attributes, String element, XMLElement parent) {
    this.tagName = tagName;
    this.attributes = attributes;
    this.element = element;
    this.parent = parent;
    this.children = new ArrayList();
  }

  public void setTagName(String tagName) { this.tagName = tagName; }
  public void setAttributes(HashMap attributes) { this.attributes = attributes; }
  public void setElement(String element) { this.element = element; }
  public void setParent(XMLElement parent) { this.parent = parent; }

  public String getTagName() { return tagName; }
  public Map getAttributes() { return attributes; }
  public String getElement() { return element; }
  public XMLElement getParent() { return parent; }
  public List getChildren() { return children; }
  public int getNumberOfChildren() { return children.size(); }

  public void addChild(XMLElement child) { child.setParent(this); children.add(child); }

  public void addChildren(Collection children) throws IllegalArgumentException {
    Object o;
    for(Iterator i = children.iterator(); i.hasNext();) {
      o = i.next();
      if(o.getClass() == XMLElement.class) { ((XMLElement)o).setParent(this); this.children.add(o); }
      else throw new IllegalArgumentException();
    }
  }

  public XMLElement getChild(int index) { return (XMLElement)children.get(index); }

  public XMLElement removeChild(int index) { return (XMLElement)children.remove(index); }

  public XMLElement getChild(String name) {
    ArrayList elts = new ArrayList();
    for(int i = 0; i < children.size(); i++) {
      if(getChild(i).getTagName().equals(name)) return getChild(i);
    }
    return null;
  }

  public XMLElement removeChild(String name) {
    ArrayList elts = new ArrayList();
    for(int i = 0; i < children.size(); i++) {
      if(getChild(i).getTagName().equals(name)) return removeChild(i);
    }
    return null;
  }

  public XMLElement[] getChildren(String name) {
    ArrayList elts = new ArrayList();
    for(int i = 0; i < children.size(); i++) {
      if(getChild(i).getTagName().equals(name)) elts.add(children.get(i));
    }
    return (XMLElement[])elts.toArray(new XMLElement[0]);
  }

  public XMLElement[] removeChildren(String name) {
    ArrayList elts = new ArrayList();
    for(int i = 0; i < children.size(); i++) {
      if(getChild(i).getTagName().equals(name)) { elts.add(children.remove(i)); i--; }
    }
    return (XMLElement[])elts.toArray(new XMLElement[0]);
  }

  public boolean hasChild() { return (!children.isEmpty()); }
  public boolean hasParent()   { return (parent != null); }

  public void print(PrintWriter pw, String indent) {
    if(hasChild()) {
      pw.print(indent + "<" + getTagName());
      if(attributes != null) {//Writes attributes:
        String[] keys = (String[])attributes.keySet().toArray(new String[0]);
        for(int i = 0; i < keys.length; i++) pw.print(" " + keys[i] + "=\"" + attributes.get(keys[i]).toString() + "\"");
      }
      pw.println(">");
      for(int i = 0; i < this.getChildren().size(); i++) {
        this.getChild(i).print(pw, indent + indentation);
      }
      pw.println(indent + "</" + getTagName() + ">");
    } else {
      pw.println(indent + "<" + getTagName() + ">" + element + "</" + getTagName() + ">");
    }
  }

  protected String tagName, element;
  protected Map attributes;
  protected XMLElement parent;
  protected List children;
  protected String indentation = "  ";
}
