// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.stats.array.ArrayOperators;
import gnu.jtools.utils.string.TextTools;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import bio.dendogram.*;
import bio.dendogram.xml.*;
import bio.dendogram.GUI.*;

public class EditionTools {

  public EditionTools(MainInterface mi, Navigator i) {
    this.mi = mi;
    this.i  = i;
    warning = new Warning(mi);
  }


  public ActionMap createEditionActions(final TreeDrawing treeDrawing) {
    ActionMap actions = new ActionMap();

    actions.put(NEW_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.Edit.NewBranch"), new ImageIcon(getClass().getResource("NewBranch.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node node = treeDrawing.getCurrentNode();
        String name = JOptionPane.showInputDialog(
            Baobab.message.getString("EditionTools.AskForLeafName"),
            "");
        if(name == null) return; // Operation cancelled.
        String[] leaves = (String[])(tree.listOfLeavesNames().toArray(new String[0]));
        if(ArrayOperators.indexOf(leaves, name) > 0) { //redondance
          Baobab.error(i, Baobab.message.getString("EditionTools.WarningDuplicate"));
          return;
        }
        try {
          Node newNode = tree.createNode(name, true);
          tree.insertNode(node, newNode);
          treeDrawing.setCurrentNode(newNode);
          tree.treeIsModified("New node added");
        } catch(NodeException ne) {
          Baobab.error(mi, ne.getMessage());
        }
      }
    });

    actions.put(NEW_MULT_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.Edit.NewMultBranch")) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node node = treeDrawing.getCurrentNode();
        String name = JOptionPane.showInputDialog(
            Baobab.message.getString("EditionTools.AskForLeafName"),
            "");
        if(name == null) return; // Operation cancelled.
        String[] leaves = (String[])(tree.listOfLeavesNames().toArray(new String[0]));
        if(ArrayOperators.indexOf(leaves, name) > 0) { //redondance
          Baobab.error(i, Baobab.message.getString("EditionTools.WarningDuplicate"));
          return;
        }
        try {
          if(!node.hasFather()) return;
          Node newNode = tree.createNode(name, true);
          tree.addNode(node.getFather(), newNode);
          tree.treeIsModified("New node added");
        } catch(NodeException ne) {
          Baobab.error(mi, ne.getMessage());
        }
      }
    });

    actions.put(NEW_SONS, new AbstractAction(Baobab.message.getString("Navigator.Edit.NewSons")) {
      public void actionPerformed(ActionEvent e) {
        ColoredTree tree = (ColoredTree)treeDrawing.getTree();
        Node node = treeDrawing.getCurrentNode();
        NewSonsDialog nsd = new NewSonsDialog(i, tree);
        nsd.setVisible(true);
        NodeList nl = nsd.getSons();
        if(nl == null || nl.size() == 0) return; //Operation cancelled.
        if(node.getNumberOfSons() + nl.size() < 2 && !mi.areSingleNodesAllowed()) {
          Baobab.error(i, Baobab.message.getString("Navigator.Edit.NewSons.NotEnoughNode"));
          return;
        }
        String[] leaves1 = (String[])(nl.listOfLeavesNames().toArray(new String[0]));
        String[] leaves2 = (String[])(tree.listOfLeavesNames().toArray(new String[0]));
        if(ArrayOperators.idElements(leaves1, leaves2).length > 0) { //redondance
          Baobab.error(i, Baobab.message.getString("EditionTools.WarningDuplicate"));
          return;
        }
        for(int i = 0; i < nl.size(); i++)
          tree.addNode(node, (Node)nl.get(i));
        tree.treeIsModified("sons added");
        i.rebuildMenus();
      }
    });

    actions.put(DELETE_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.EditDel"), new ImageIcon(getClass().getResource("DeleteBranch.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        if(currentNode == tree.getRootNode()) return;

        boolean continu = true;
        if(!treeDrawing.isEdited() && (TreeTools.hasAtLeastOneLength(tree) || TreeTools.hasAtLeastOneBootstrap(tree))) {//premire dition
          continu = warningAboutEdition();
        }
        if(continu) {
          NavigatorRemote.moveBackward(treeDrawing);
          tree.removeNode(currentNode);
          mi.rebuildMenu();
          tree.treeIsModified("A branch has been deleted.");
        }
      }
    });

    actions.put(CUT_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.EditCut"), new ImageIcon(getClass().getResource("CutBranch.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        if(currentNode == tree.getRootNode()) return;

        boolean continu = true;
        if(!treeDrawing.isEdited() && (TreeTools.hasAtLeastOneLength(tree) || TreeTools.hasAtLeastOneBootstrap(tree))) {//premire dition
          continu = warningAboutEdition();
        }
        if(continu) {
          //Must check for length first.
          //If currentNode is a son of root, we must add lengths:
          if( currentNode.getFather() == tree.getRootNode()
           && tree.getRootNode().getNumberOfSons() == 2
           && tree.getRootNode().getSon(0).hasLength()
           && tree.getRootNode().getSon(1).hasLength()
            ) {
            currentNode.setLength( tree.getRootNode().getSon(0).getLengthValue()
                                 + tree.getRootNode().getSon(1).getLengthValue()
                                 );
          }
          String saveName = Baobab.request(i, Baobab.message.getString("EditionTools.AskForTreeName"), "");
          if(saveName != null) {
            if(ArrayOperators.indexOf(mi.getList().getSavedNames(), saveName) > -1) {
              Baobab.error(i, Baobab.message.getString("EditionTools.BadTreeName"));
            } else {
              treeDrawing.setCurrentNode(currentNode.getFather()); // Must do this first! (reference to father will then be deleted)
              tree.removeNode(currentNode);
              ColoredTree.setColorsOfAllSubtrees((ColoredNode)currentNode, ColoredNode.COLORS_DEFAULT);
              String format = mi.getPreferredFormatOfCurrentTree();
              mi.addTreeToList(new ColoredTree(saveName, (ColoredNode)currentNode), saveName, format);
              mi.rebuildMenu();
              tree.treeIsModified("A branch has been cut");
            }
          }
        }
      }
    });

    actions.put(COPY_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.EditCopy"), new ImageIcon(getClass().getResource("CopyBranch.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        ColoredNode branch = (ColoredNode)ColoredTree.cloneSubtree(currentNode);

        String saveName = Baobab.request(i, Baobab.message.getString("EditionTools.AskForCopyName"), "");
        if(saveName != null) {
          if(ArrayOperators.indexOf(mi.getList().getSavedNames(), saveName)>-1) {
            Baobab.error(i, Baobab.message.getString("Navigator.BadTreeName"));
          } else {
            String format = mi.getPreferredFormatOfCurrentTree();
            mi.addTreeToList(new ColoredTree(saveName, branch), saveName, format);
            mi.rebuildMenu();
          }
        }
      }
    });

    actions.put(ADD_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.EditAddBranch")) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        int n = choseTree();
        boolean continu = true;
        if(n > -1) {
          if(!treeDrawing.isEdited() && (TreeTools.hasAtLeastOneLength(tree) || TreeTools.hasAtLeastOneBootstrap(tree))) {//premire dition
            continu = warningAboutEdition();
          }
          if(continu) {
            String[] leaves1 = (String[])(mi.getList().treeAt(n).listOfLeavesNames().toArray(new String[0]));
            String[] leaves2 = (String[])(tree.listOfLeavesNames().toArray(new String[0]));
            boolean continu2 = true;
            if(ArrayOperators.idElements(leaves1, leaves2).length > 0) {//redondance
              Baobab.error(i, Baobab.message.getString("EditionTools.WarningDuplicate"));
              continu2 = false;
            }
            if(continu2) {
              tree.addNode(currentNode, (((ColoredTree)mi.getList().treeAt(n).clone()).getRootNode()));
              mi.rebuildMenu();
              tree.treeIsModified("A branch has been added.");
            }
          }
        }
      }
    });

    actions.put(INSERT_BRANCH, new AbstractAction(Baobab.message.getString("Navigator.EditAddNode"), new ImageIcon(getClass().getResource("PasteBranch.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        TreePath path = tree.getTreePath(currentNode);
        int n = choseTree();
        boolean continu = true;
        if(n > -1) {
          if(!treeDrawing.isEdited() && (TreeTools.hasAtLeastOneLength(tree) || TreeTools.hasAtLeastOneBootstrap(tree))) {//premire dition
            continu = warningAboutEdition();
          }
          if(continu) {
            String[]leaves1 = (String[])(mi.getList().treeAt(n).listOfLeavesNames().toArray(new String[0]));
            String[]leaves2 = (String[])(mi.getCurrentTree()   .listOfLeavesNames().toArray(new String[0]));
            boolean continu2 = true;
            if(ArrayOperators.idElements(leaves1, leaves2).length>0) {//redondance
              Baobab.error(i, Baobab.message.getString("EditionTools.WarningDuplicate"));
              continu2 = false;
            }
            if(continu2) {
              tree.insertNode(((ColoredTree)(mi.getList().treeAt(n).clone())).getRootNode(), path);
              i.getTreeDrawing().setCurrentNode(path);
              tree.treeIsModified("A node has been added.");
            }
          }
        }
      }
    });

    actions.put(SWAP_BRANCHES, new AbstractAction(Baobab.message.getString("Navigator.DisplaySwap"), new ImageIcon(getClass().getResource("SwapBranches.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        int n = currentNode.getNumberOfSons(); int n1 = 0, n2 = 1;
        if(n > 2) {//Asks for branches to swap:
          String text = Baobab.request(i, Baobab.message.getString("Navigator.SwapChoice"), "");
          if (text != null) {
            StringTokenizer ana = new StringTokenizer(text, ",");
            if (ana.countTokens() == 2) {
              try {
                n1 = Integer.parseInt(ana.nextToken()) - 1;
                n2 = Integer.parseInt(ana.nextToken()) - 1;
              }
              catch (NumberFormatException ex) {
                Baobab.error(i, Baobab.message.getString("Navigator.NotAValidNumber"));
              }
            }
            else { //nombre insuffisant de numros, ou trop de nombres.
              Baobab.error(i, Baobab.message.getString("Navigator.ComaError"));
            }
          }
        }
        try {
          currentNode.swap(n1, n2);
        } catch (IndexOutOfBoundsException ex) { //paramtres incorrects
          Baobab.error(i, Baobab.message.getString("Navigator.BranchDoesNotExist"));
        }
        tree.treeIsModified("Branches swaped.");
      }
    });

    actions.put(REROOT_TREE, new AbstractAction(Baobab.message.getString("Navigator.DisplayReRoot"), new ImageIcon(getClass().getResource("NewOutgroup.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        //if(mi.getNumberOfNavigators() > 1) {
        //  error(message.getString("EditionToolsCloseNavigators"));
        if(currentNode == tree.getRootNode()) return;
        else {
          try {
            tree.newOutGroup(currentNode);
          } catch(NodeException ne) {
            Baobab.warning(i, Baobab.message.getString("Navigator.Reroot.WarningLengthMissing"));
          }

          mi.returnToRoot();
          tree.treeIsModified("Tree re-rooted.");
          mi.rebuildMenu();
        }
      }
    });

    actions.put(SET_LENGTH, new AbstractAction(Baobab.message.getString("Navigator.EditSetLength"), new ImageIcon(getClass().getResource("SetLength.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        String value = Baobab.request(i, Baobab.message.getString("EditionTools.RequestLength"),
                              currentNode.hasLength() ? currentNode.getLength().toString() : "");
        try {
          //try {
            //((XNode) currentNode).setLength(Double.parseDouble(value), i.doesAllowValuesModifications());
            ((XNode) currentNode).setLength(Double.parseDouble(value), true);
          //} catch(ClassCastException ex) {
          //  currentNode.setLength(Double.parseDouble(value));
          //}
          tree.treeIsModified("Lengths have changed.");
          mi.rebuildAllCurrentNavigatorMenus();
          mi.redrawAllCurrentNavigators();
          mi.rebuildMenu();
        } catch(NumberFormatException ex) { Baobab.error(i, Baobab.message.getString("badNumber"));
        } catch(NullPointerException ex) { //cancel.
        }
      }
    });

    actions.put(SET_BOOTSTRAP, new AbstractAction(Baobab.message.getString("Navigator.EditSetBootstrap"), new ImageIcon(getClass().getResource("SetBootstrap.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        String value = Baobab.request(i, Baobab.message.getString("EditionTools.RequestBootstrap"), currentNode.getBootstrap().toString());
        try {
          //try {
            //((XNode)currentNode).setBootstrap(Double.parseDouble(value), i.doesAllowValuesModifications());
            ((XNode)currentNode).setBootstrap(Double.parseDouble(value), true);
          //} catch(ClassCastException ex) {
          //  currentNode.setBootstrap(Double.parseDouble(value));
          //}
          mi.getCurrentTree().treeIsModified("Bootstraps have changed.");
          mi.rebuildAllCurrentNavigatorMenus();
          mi.redrawAllCurrentNavigators();
          mi.rebuildMenu();
        } catch(NumberFormatException ex) { Baobab.error(i, Baobab.message.getString("badNumber"));
        } catch(NullPointerException ex) { //cancel.
        }
      }
    });

    actions.put(DELETE_LENGTH, new AbstractAction(Baobab.message.getString("Navigator.EditDeleteLength"), new ImageIcon(getClass().getResource("DeleteLength.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        currentNode.deleteLength();
        tree.treeIsModified("A length has been deleted.");
        mi.rebuildAllCurrentNavigatorMenus();
        mi.rebuildMenu();
      }
    });

    actions.put(DELETE_BOOTSTRAP, new AbstractAction(Baobab.message.getString("Navigator.EditDeleteBootstrap"), new ImageIcon(getClass().getResource("DeleteBootstrap.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        currentNode.deleteBootstrap();
        tree.treeIsModified("A bootstrap has been deleted.");
        mi.rebuildAllCurrentNavigatorMenus();
        mi.rebuildMenu();
      }
    });

    actions.put(DELETE_NAME, new AbstractAction(Baobab.message.getString("Navigator.Edit.DeleteName")) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        currentNode.deleteName();
        tree.treeIsModified("Inner node name deleted.");
        mi.rebuildAllCurrentNavigatorMenus();
      }
    });

    actions.put(SET_NAME, new AbstractAction(Baobab.message.getString("Navigator.Edit.SetName"), new ImageIcon(getClass().getResource("RenameNode.gif"))) {
      public void actionPerformed(ActionEvent e) {
        Tree tree = treeDrawing.getTree();
        Node currentNode = treeDrawing.getCurrentNode();
        String oldName = currentNode.getName();
        String newName = Baobab.request(i, Baobab.message.getString("EditionTools.AskForNodeName"), oldName);
        if(newName != null) {
          newName = TextTools.removeFirstAndLastBlanks(newName);
          if(newName.equals(oldName)) return; //Operation cancelled.
          if(new StringTokenizer(newName," ").countTokens()>0) {//nom correct
            String[] feuilles = (String[])(tree.listOfLeavesNames().toArray(new String[0]));
            if(currentNode.isLeaf() && ArrayOperators.indexOf(feuilles, newName) > -1) {
              Baobab.error(i, Baobab.message.getString("Navigator.WarningDuplicate"));
            } else {
              String[] groupes = (String[])(GroupsManager.listOfGroupsNames(tree).toArray(new String[0]));
              if(ArrayOperators.indexOf(groupes, newName)>-1) {
                Baobab.error(i, Baobab.message.getString("EditionTools.WarningGpDup"));
              } else {
                currentNode.setName(newName);
                tree.treeIsModified("A name has changed.");
                mi.rebuildMenu();
                mi.rebuildAllCurrentNavigatorMenus();
              }
            }
          } else Baobab.error(i, Baobab.message.getString("Navigator.VoidGroupName"));
        }
      }
    });

    actions.put(TRUNC_LEAVES_NAMES, new AbstractAction(Baobab.message.getString("EditionTools.TruncLeavesNames")) {
      public void actionPerformed(ActionEvent e) {
        String size = Baobab.request(i, Baobab.message.getString("EditionTools.TruncLeavesNamesRequest"), "10");
        try {
          int max = Integer.parseInt(size);
          TreeTools.truncLeavesNames(i.getCurrentNode(), max);
          mi.redrawAllCurrentNavigators();
          mi.redrawMainInterface();
        } catch(Exception ex) {
          Baobab.error(i, Baobab.message.getString("badNumber"));
        }
      }
    });

    actions.put(REMOVE_BLANKS_LEAVES_NAMES, new AbstractAction(Baobab.message.getString("EditionTools.RemoveBlanksFromLeavesNames")) {
      public void actionPerformed(ActionEvent e) {
        TreeTools.correctNames(i.getCurrentNode(), " \t\n\r\f");
        mi.redrawAllCurrentNavigators();
        mi.redrawMainInterface();
      }
    });

    actions.put(CORRECT_NAMES, new AbstractAction(Baobab.message.getString("EditionTools.CorrectNames")) {
      public void actionPerformed(ActionEvent e) {
        String chars = Baobab.request(i, Baobab.message.getString("EditionTools.CorrectNames.GetChars"), "_@");
        if(chars == null) return; // Operation cancelled.
        TreeTools.correctNames(i.getCurrentNode(), chars);
        mi.redrawAllCurrentNavigators();
        mi.redrawMainInterface();
      }
    });

    actions.put(TRANSLATE_NAMES, new AbstractAction(Baobab.message.getString("EditionTools.TranslateNames")) {
      public void actionPerformed(ActionEvent e) {
        if(tmld == null) tmld = new TranslationMapLoadDialog(i);
        tmld.setVisible(true);
        if(tmld.getAction().equals(tmld.CANCEL)) return; //Operation cancelled.
        try {
          Map translation = TreeTools.loadTranslationMapFromFile(tmld.getFile(),
              tmld.getNumberOfLinesToDrop(), tmld.getSeparator());
          TreeTools.translateNames(i.getCurrentNode(), translation);
          mi.redrawAllCurrentNavigators();
          mi.redrawMainInterface();
        } catch(IOException ioe) {
          Baobab.error(i, Baobab.message.getString("EditionTools.TranslateNames.IOError"));
        }
      }
    });

    actions.put(REVERSE_NAMES, new AbstractAction(Baobab.message.getString("EditionTools.ReverseNames")) {
      public void actionPerformed(ActionEvent e) {
        if(tmld == null) tmld = new TranslationMapLoadDialog(i);
        tmld.setVisible(true);
        if(tmld.getAction().equals(tmld.CANCEL)) return; //Operation cancelled.
        try {
          Map translation = TreeTools.loadTranslationMapFromFile(tmld.getFile(),
              tmld.getNumberOfLinesToDrop(), tmld.getSeparator());
          TreeTools.translateNames(i.getCurrentNode(), TreeTools.reverseTranslation(translation));
          mi.redrawAllCurrentNavigators();
          mi.redrawMainInterface();
        } catch(IOException ioe) {
          Baobab.error(i, Baobab.message.getString("EditionTools.TranslateNames.IOError"));
        }
      }
    });

    actions.put(UNRESOLVE, new AbstractAction(Baobab.message.getString("EditionTools.Unresolve")) {
      public void actionPerformed(ActionEvent e) {
        TreeTools.unresolve(i.getCurrentNode());
        mi.redrawAllCurrentNavigators();
        mi.redrawMainInterface();
      }
    });

    actions.put(DEEPUNRESOLVE, new AbstractAction(Baobab.message.getString("EditionTools.DeepUnresolve")) {
      public void actionPerformed(ActionEvent e) {
        TreeTools.deepUnresolve(i.getCurrentNode());
        mi.redrawAllCurrentNavigators();
        mi.redrawMainInterface();
      }
    });

    return actions;
  }

  private int choseTree() {
    StockTree list1 = mi.getList();
    StockTree list2 = mi.getOtherTrees();
    int result=-1;
    String select=(String)(JOptionPane.showInputDialog(i,
      Baobab.message.getString("EditionTools.SelectTree"),
      Baobab.message.getString("EditionTools.InsertTree"),
      JOptionPane.QUESTION_MESSAGE,
      null,
      list2.getSavedNames(),
      list2.getNameAt(0)));
    if(select != null) result = list1.whoseNameIsIt(select);
    return result;
  }

  private boolean warningAboutEdition() {
    warning.setVisible(true);
    return warning.getContinu();
  }

  private boolean continu(String text) {
    boolean result = false;
    int choix = JOptionPane.showConfirmDialog(i, text, Baobab.message.getString("warning"), JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
    if(choix == JOptionPane.OK_OPTION) result = true;
    return result;
  }


  private MainInterface mi;
  private Navigator i;
  private Warning warning;
  private TranslationMapLoadDialog tmld;

  public static final String NEW_BRANCH = "new branch";
  public static final String NEW_MULT_BRANCH = "new multifurcating branch";
  public static final String NEW_SONS = "new sons";
  public static final String DELETE_BRANCH = "delete branch";
  public static final String CUT_BRANCH = "cut branch";
  public static final String COPY_BRANCH = "copy branch";
  public static final String ADD_BRANCH = "add branch";
  public static final String INSERT_BRANCH = "insert branch";
  public static final String SWAP_BRANCHES = "swap branches";
  public static final String SET_LENGTH = "set length";
  public static final String DELETE_LENGTH = "delete length";
  public static final String SET_BOOTSTRAP = "set bootstrap";
  public static final String DELETE_BOOTSTRAP = "delete botstrap";
  public static final String SET_NAME = "set name";
  public static final String DELETE_NAME = "delete name";
  public static final String REROOT_TREE = "reroot tree";
  public static final String TRUNC_LEAVES_NAMES = "trunc leaves names";
  public static final String REMOVE_BLANKS_LEAVES_NAMES = "remove blanks from leaves names";
  public static final String CORRECT_NAMES = "correct names";
  public static final String TRANSLATE_NAMES = "translate names";
  public static final String REVERSE_NAMES = "reverse names";
  public static final String UNRESOLVE = "unresolve node";
  public static final String DEEPUNRESOLVE = "deep unresolve node";
}





/**
 * Warning class: an JDialog displayed when a tree is about to be edited.
 */

class Warning extends JDialog implements ActionListener{

  public Warning(MainInterface mi) {
    super(mi, true);
    super.setBounds(  mi.getX()+(mi.getWidth() -300)/2,//centree en largeur
                      mi.getY()+(mi.getHeight()-160 )/2,//centree en hauteur
                        300,
                        160);
    this.mi = mi;
    setTitle(Baobab.message.getString("warningEdition"));
    keepLengths    = new JCheckBox(Baobab.message.getString("warningKeepLengths"));
    keepBootstraps = new JCheckBox(Baobab.message.getString("warningKeepBootstraps"));
    ok             = new Button  (Baobab.message.getString("ok"));
      ok.addActionListener(this);
    cancel         = new Button  (Baobab.message.getString("cancel"));
      cancel.addActionListener(this);
    warningContent = super.getContentPane();
    g = new GridBagLayout();
    warningContent.setLayout(g);
    c = new GridBagConstraints();

      c.gridx=0;      c.gridy=0;
      c.gridwidth=2;  c.gridheight=1;
      c.weightx=0;    c.weighty=10;
    warningContent.add(new Label(Baobab.message.getString("warningParametersFound")),c);
      c.gridx=0;      c.gridy=1;
      c.gridwidth=2;  c.gridheight=1;
      c.weightx=0;    c.weighty=10;
    warningContent.add(new Label(Baobab.message.getString("warningParametersChoice")),c);
      c.gridx=0;      c.gridy=2;
      c.gridwidth=2;  c.gridheight=1;
      c.weightx=0;    c.weighty=10;
    warningContent.add(keepLengths,c);
      c.gridx=0;      c.gridy=3;
      c.gridwidth=2;  c.gridheight=1;
      c.weightx=0;    c.weighty=10;
    warningContent.add(keepBootstraps,c);
      c.gridx=0;      c.gridy=4;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=50;   c.weighty=0;
    warningContent.add(ok,c);
      c.gridx=1;      c.gridy=4;
      c.gridwidth=1;  c.gridheight=1;
      c.weightx=50;   c.weighty=10;
    warningContent.add(cancel,c);
  }

  public boolean getContinu() { return continu; }

  public void actionPerformed(ActionEvent ev) {
    Object source = ev.getSource();
    if(source == ok) {
      //on efface ce qui est spcifi:
      if(!   keepLengths.isSelected()) (mi.getCurrentTree()).deleteLengthsOfTree();
      if(!keepBootstraps.isSelected()) (mi.getCurrentTree()).deleteBootstrapsOfTree();
      continu = true;
//      mi.nowEdited();
      setVisible(false);
    }
    if(source == cancel) {
      continu = false;
      setVisible(false);
    }
  }

  private MainInterface mi;
  private boolean continu;
  private Button ok, cancel;
  private JCheckBox keepLengths, keepBootstraps;
  private Container warningContent;
  private GridBagLayout g;
  private GridBagConstraints c;
}
