// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.utils.gui.GraphicsTools;
import gnu.jtools.utils.file.FileTools;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class FileEditor extends JDialog {

  public FileEditor(Frame owner, File file) {
    super(owner);
    setFile(file);
    initActions();
    initGUI();
    loadFromFile(file);
    pack();
    GraphicsTools.center(this);
  }

  protected void initGUI() {
    editor = new JTextArea();

    toolBar = new JToolBar("File", JToolBar.HORIZONTAL);
    toolBar.add(save);
    toolBar.add(reload);

    // Setting layout:
    Container cp = this.getContentPane();
    cp.setLayout(new BorderLayout());
    cp.add(new JScrollPane(editor), BorderLayout.CENTER);
    cp.add(toolBar, BorderLayout.NORTH);
  }

  protected void initActions() {
    save = new AbstractAction(Baobab.message.getString("FileEditor.save")) {
      public void actionPerformed(ActionEvent e) {
        saveToFile(file);
      }
    };

    reload = new AbstractAction(Baobab.message.getString("FileEditor.reload")) {
      public void actionPerformed(ActionEvent e) {
        loadFromFile(file);
      }
    };
  }

  public void loadFromFile(File file) {
    try {
      FileReader fr = new FileReader(file);
      BufferedReader br = new BufferedReader(fr);
      String line = br.readLine();
      if(line != null) editor.setText(line); // This clears the editor too.
      line = br.readLine();
      while(line != null) {
        editor.append("\n" + line);
        line = br.readLine();
      }
      br.close();
    } catch(IOException ioe) {
      JOptionPane.showMessageDialog(
          this,
          Baobab.message.getString("badPath"),
          Baobab.message.getString("error"),
          JOptionPane.ERROR_MESSAGE);
    }
  }

  public void saveToFile(File file) {
    try {
      FileTools.copyTo(file, new File(file.getParent(), file.getName() + ".bck"));
      FileWriter fw = new FileWriter(file);
      PrintWriter pw = new PrintWriter(fw);
      pw.println(editor.getText());
      pw.close();
    } catch(IOException ioe) {
      JOptionPane.showMessageDialog(
          this,
          Baobab.message.getString("badPath"),
          Baobab.message.getString("error"),
          JOptionPane.ERROR_MESSAGE);
    }
  }

  public void setFile(File file) { this.file = file; }

  public File getFile() { return file; }

  File file;

  // GUI components:
  protected JTextArea editor;
  protected JToolBar toolBar;
  protected Action save, reload;

}
