// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

package TreeEditor;

import gnu.jtools.utils.gui.GraphicsTools;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import bio.dendogram.*;

/**
 * <p>Titre : Tree Editor</p>
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * <p>Socit : </p>
 * @author Julien Dutheil
 * @version 1.0
 */

// This is a modified version of the ListDemo.java file from the Java Tutorial.

public class NewSonsDialog extends JDialog implements ListSelectionListener {

  private ColoredTree tree;
  private JList list;
  private DefaultListModel listModel;

  private static final String hireString = "Hire";
  private static final String fireString = "Fire";
  private JButton fireButton, hireButton, okButton;
  private JTextField nodeName;

  public NewSonsDialog(Frame owner, ColoredTree tree) {
    super(owner, Baobab.message.getString("Navigator.Edit.NewSons"), true);
    this.tree = tree;

    listModel = new DefaultListModel();

    //Create the list and put it in a scroll pane.
    list = new JList(listModel);
    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    list.setSelectedIndex(0);
    list.addListSelectionListener(this);
    list.setVisibleRowCount(5);
    JScrollPane listScrollPane = new JScrollPane(list);

    hireButton = new JButton(hireString);
    HireListener hireListener = new HireListener(hireButton);
    hireButton.setActionCommand(hireString);
    hireButton.addActionListener(hireListener);

    fireButton = new JButton(fireString);
    fireButton.setActionCommand(fireString);
    fireButton.addActionListener(new FireListener());
    fireButton.setEnabled(false);

    nodeName = new JTextField(10);
    nodeName.addActionListener(hireListener);
    nodeName.getDocument().addDocumentListener(hireListener);

    okButton = new JButton("ok");
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
    //Create a panel that uses BoxLayout.
    JPanel buttonPane = new JPanel();
    buttonPane.setLayout(new BoxLayout(buttonPane,
                                       BoxLayout.LINE_AXIS));
    buttonPane.add(fireButton);
    buttonPane.add(Box.createHorizontalStrut(5));
    buttonPane.add(new JSeparator(SwingConstants.VERTICAL));
    buttonPane.add(Box.createHorizontalStrut(5));
    buttonPane.add(nodeName);
    buttonPane.add(hireButton);
    buttonPane.add(okButton);
    buttonPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(listScrollPane, BorderLayout.CENTER);
    this.getContentPane().add(buttonPane, BorderLayout.PAGE_END);
    pack();
    GraphicsTools.center(this);
  }

  class FireListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      //This method can be called only if
      //there's a valid selection:
      int index = list.getSelectedIndex();
      if(index == -1) return;

      //so go ahead and remove whatever's selected.
      listModel.remove(index);

      int size = listModel.getSize();

      if (size == 0) { //Nobody's left, disable firing.
        fireButton.setEnabled(false);

      } else { //Select an index.
        if (index == listModel.getSize()) {
          //removed item in last position
          index--;
        }

        list.setSelectedIndex(index);
        list.ensureIndexIsVisible(index);
      }
    }
  }

  //This listener is shared by the text field and the hire button.
  class HireListener implements ActionListener, DocumentListener {
    private boolean alreadyEnabled = false;
    private JButton button;

    public HireListener(JButton button) {
      this.button = button;
    }

    //Required by ActionListener.
    public void actionPerformed(ActionEvent e) {
      String name = nodeName.getText();

      //User didn't type in a unique name...
      if (name.equals("") || alreadyInList(name)) {
        Toolkit.getDefaultToolkit().beep();
        nodeName.requestFocusInWindow();
        nodeName.selectAll();
        return;
      }

      listModel.addElement(nodeName.getText());

      //Reset the text field.
      nodeName.requestFocusInWindow();
      nodeName.setText("");

      //Select the new item and make it visible.
      int index = listModel.size();
      list.setSelectedIndex(index);
      list.ensureIndexIsVisible(index);
    }

    //This method tests for string equality. You could certainly
    //get more sophisticated about the algorithm.  For example,
    //you might want to ignore white space and capitalization.
    protected boolean alreadyInList(String name) {
      return listModel.contains(name);
    }

    //Required by DocumentListener.
    public void insertUpdate(DocumentEvent e) {
      enableButton();
    }

    //Required by DocumentListener.
    public void removeUpdate(DocumentEvent e) {
      handleEmptyTextField(e);
    }

    //Required by DocumentListener.
    public void changedUpdate(DocumentEvent e) {
      if (!handleEmptyTextField(e)) {
        enableButton();
      }
    }

    private void enableButton() {
      if (!alreadyEnabled) {
        button.setEnabled(true);
      }
    }

    private boolean handleEmptyTextField(DocumentEvent e) {
      if (e.getDocument().getLength() <= 0) {
        button.setEnabled(false);
        alreadyEnabled = false;
        return true;
      }
      return false;
    }
  }

  //This method is required by ListSelectionListener.
  public void valueChanged(ListSelectionEvent e) {
    if (e.getValueIsAdjusting() == false) {

      if (list.getSelectedIndex() == -1) {
        //No selection, disable fire button.
        fireButton.setEnabled(false);

      } else {
        //Selection, enable the fire button.
        fireButton.setEnabled(true);
      }
    }
  }

  // Method for retrieving nodes:
  public NodeList getSons() {
    NodeList nl = new NodeList(listModel.size());
    try {
      for(int i = 0; i < listModel.size(); i++)
        nl.add(tree.createNode( (String) listModel.elementAt(i), true));
    } catch(NodeException ne) {
      Baobab.error(this, ne.getMessage());
      return null;
    }

    return nl;
  }

}
