// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import gnu.jtools.postscript.PostScriptFilter;
import gnu.jtools.utils.file.FileTools;
import bio.dendogram.*;
import bio.dendogram.GUI.*;
import bio.dendogram.print.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.awt.print.*;

class PrintDialog extends JDialog implements ActionListener{

  public PrintDialog(MainInterface mi, File initialPath) {
    super(mi, true);
    this.mi = mi;
    setLocale(mi.getLocale());
    setTitle(Baobab.message.getString("MainInterface.TreePrint"));
    pageFormat = new PageFormat();

    //Layout manager: border layout par dfaut.
    dialogContent = getContentPane();

    Box vbox, hbox;

    //Paneau de spcification des paramtres:
    panel1 = new JPanel();
    vbox = Box.createVerticalBox();
    JLabel label1 = new JLabel(Baobab.message.getString("printDialogTreeParam"));
    vbox.add(label1);
    printWholeTree = new JRadioButton(Baobab.message.getString("printDialogPrintWholeTree"), true);
    printWholeTree.addActionListener(this);
    vbox.add(printWholeTree);
    printGroups = new JRadioButton(Baobab.message.getString("printDialogPrintGroups"));
    printGroups.addActionListener(this);
    vbox.add(printGroups);
    ButtonGroup bg = new ButtonGroup();
    bg.add(printWholeTree);
    bg.add(printGroups);
    printLengths = new JCheckBox(Baobab.message.getString("printDialogPrintLengths"));
    vbox.add(printLengths);
    printBootstraps = new JCheckBox(Baobab.message.getString("printDialogPrintBootstraps"));
    vbox.add(printBootstraps);
    JLabel label2 = new JLabel(Baobab.message.getString("printDialogNbMax"));
    vbox.add(label2);
    setNbMax = new JTextField("50", 5);
    vbox.add(setNbMax);
    panel1.add(vbox);

    vbox = Box.createVerticalBox();
    JLabel label3 = new JLabel(Baobab.message.getString("printDialogPrintTo"));
    vbox.add(label3);
    printInPS = new JRadioButton(Baobab.message.getString("printDialogPrintInPS"), true);//par dfaut.
    printInPS.addActionListener(this);
    vbox.add(printInPS);
    printOnPrinter = new JRadioButton(Baobab.message.getString("printDialogPrintOnPrinter"));
    printOnPrinter.addActionListener(this);
    vbox.add(printOnPrinter);
    ButtonGroup bg2 = new ButtonGroup();
    bg2.add(printInPS);
    bg2.add(printOnPrinter);
    chooseAFileLabel = new JLabel(Baobab.message.getString("dialogChooseAFile"));
    vbox.add(chooseAFileLabel);
    chooseAFile = new JTextField(30);
    vbox.add(chooseAFile);
    browseFile = new JButton(Baobab.message.getString("dialogBrowse"));
    browseFile.addActionListener(this);
    vbox.add(browseFile);
    chooseADirectoryLabel = new JLabel(Baobab.message.getString("dialogChooseADirectory"));
    vbox.add(chooseADirectoryLabel);
    chooseADirectory = new JTextField(30);
    vbox.add(chooseADirectory);
    browseDirectory = new JButton(Baobab.message.getString("dialogBrowse"));
    browseDirectory.addActionListener(this);
    vbox.add(browseDirectory);
    panel1.add(vbox);
    panel1.setBorder(
      BorderFactory.createTitledBorder(
        BorderFactory.createLineBorder(Color.black),
        Baobab.message.getString("printDialog")));;

    dialogContent.add(panel1, BorderLayout.EAST);

    hbox = Box.createHorizontalBox();
    ok = new JButton(Baobab.message.getString("ok"));
    ok.addActionListener(this);
    ok.setMnemonic(Baobab.message.getString("cancelMnemo").toCharArray()[0]);
    hbox.add(ok);
    cancel = new JButton(Baobab.message.getString("cancel"));
    cancel.addActionListener(this);
    cancel.setMnemonic(Baobab.message.getString("cancelMnemo").toCharArray()[0]);
    hbox.add(cancel);
    dialogContent.add(hbox, BorderLayout.SOUTH);

    output_fc = new JFileChooser();
    output_fc.addActionListener(this);
    output_fc.setApproveButtonText("Print");
    output_fc.setApproveButtonMnemonic('P');
    output_fc.addChoosableFileFilter(psFilter);
    output_fc.setLocale(mi.getLocale());
    output_dc = new JFileChooser(initialPath);
    output_dc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    output_dc.setLocale(mi.getLocale());
    if(initialPath != null){
      output_fc.setCurrentDirectory(initialPath);
      output_dc.setCurrentDirectory(initialPath);
    }

    pack();
    Toolkit tk=Toolkit.getDefaultToolkit();
    setBounds((tk.getScreenSize().width-getWidth())/2, (tk.getScreenSize().height-getHeight())/2, getWidth(), getHeight());
  }

  private void actualize() {
    this.tree=(ColoredTree)(mi.getCurrentTree().clone());

    //Does this tree has lengths?
    if(TreeTools.hasAtLeastOneLength(tree)) printLengths.setEnabled(true);
    else {
      printLengths.setSelected(false);
      printLengths.setEnabled(false);
    }

    //Cet arbre a-t-il des bootstraps?
    if(TreeTools.hasAtLeastOneBootstrap(tree)) printBootstraps.setEnabled(true);
    else {
      printBootstraps.setSelected(false);
      printBootstraps.setEnabled(false);
    }

    //Imprimer sur l'imprimante ou dans un fichier?
    if(printOnPrinter.isSelected()) {
      chooseADirectoryLabel.setEnabled(false);
      chooseADirectory.setEnabled(false);
      browseDirectory.setEnabled(false);
      chooseAFileLabel.setEnabled(false);
      chooseAFile.setEnabled(false);
      browseFile.setEnabled(false);
    } else {
      //Imprimer l'arbre entier ou les groupes sparments?
      if(printWholeTree.isSelected()) {
        chooseADirectoryLabel.setEnabled(false);
        chooseADirectory.setEnabled(false);
        browseDirectory.setEnabled(false);
        chooseAFileLabel.setEnabled(true);
        chooseAFile.setEnabled(true);
        browseFile.setEnabled(true);
      } else {
        chooseADirectoryLabel.setEnabled(true);
        chooseADirectory.setEnabled(true);
        browseDirectory.setEnabled(true);
        chooseAFileLabel.setEnabled(false);
        chooseAFile.setEnabled(false);
        browseFile.setEnabled(false);
      }
    }
    //Toutes les informations ont-elles t fournies?
    ok.setEnabled(false);
    if(     printInPS.isSelected() && printWholeTree.isSelected() && !     chooseAFile.getText().equals("")) ok.setEnabled(true);
    if(     printInPS.isSelected() &&    printGroups.isSelected() && !chooseADirectory.getText().equals("")) ok.setEnabled(true);
    if(printOnPrinter.isSelected()) ok.setEnabled(true);
  }


  private void printTree(ColoredTree tree) throws NumberFormatException {
    //Imprime un arbre, sur une ou plusieurs pages.
    int nMax=Integer.parseInt(setNbMax.getText());
    TreeDrawing treeDrawing = (TreeDrawing)mi.getTreeDrawing().clone();
                treeDrawing.setDrawBranchesNumbers(false);
                treeDrawing.setDrawLengths(printLengths.isSelected());
                treeDrawing.setDrawBootstraps(printBootstraps.isSelected());
    tp = new TreePaginater(treeDrawing, nMax, pageFormat);
    if(printInPS.isSelected()) {//Impression dans un fichier .PS:
      String name = FileTools.correctFilename(tree.getName())+".ps";
      File newFile = new File(chooseADirectory.getText(), name);
      boolean continu = true;
      if(newFile.exists()) continu = continu(newFile.getName() + " " + Baobab.message.getString("alreadyExists"));
      try { if(continu) tp.printAsPostScript(newFile); }
      catch(IOException e) { e.printStackTrace(); }
    }
    if(printOnPrinter.isSelected()) {//Impression sur l'imprimante:
      printJob.setPageable(tp);
      try { printJob.print();}
      catch(PrinterException e) { e.printStackTrace(); }
    }
  }
  private void printTree(ColoredTree tree, File newFile) throws NumberFormatException {
    //Print a tree (one or more pages).
    int nMax=Integer.parseInt(setNbMax.getText());
    //Clone the tree drawing:
    ColoredTreeDrawing treeDrawing = (ColoredTreeDrawing)mi.getTreeDrawing().clone();
                treeDrawing.setDrawBranchesNumbers(false);
                treeDrawing.setDrawLengths(printLengths.isSelected());
                treeDrawing.setDrawBootstraps(printBootstraps.isSelected());
    tp = new TreePaginater(treeDrawing, nMax, pageFormat);
    if(printInPS.isSelected()) {//Impression dans un fichier .PS:
      boolean continu = true;
      if(newFile.exists()) continu = continu(newFile.getName() + " " + Baobab.message.getString("alreadyExists"));
      try { if(continu) tp.printAsPostScript(newFile); }
      catch(IOException e) { e.printStackTrace(); }
    }
    if(printOnPrinter.isSelected()) {//Impression sur l'imprimante:
      printJob.setPageable(tp);
      try { printJob.print();}
      catch(PrinterException e) { e.printStackTrace(); }
    }
  }

  public void actionPerformed(ActionEvent ev) {
    actualize();
    Object source=ev.getSource();

    if(source==browseFile) {
      output_fc.showSaveDialog(this);
      if(output_fc.getSelectedFile()!=null) {
        chooseAFile.setText(output_fc.getSelectedFile().getAbsolutePath());
        actualize();
      }
    }
    if(source==browseDirectory) {
      output_dc.showSaveDialog(this);
      if(output_dc.getSelectedFile() != null) {
        chooseADirectory.setText(output_dc.getSelectedFile().getAbsolutePath());
        actualize();
      }
    }

    if(source == ok) {
      //Choix effectu une seule fois pour tous les fichiers.
      if(printOnPrinter.isSelected()) {
        printJob = PrinterJob.getPrinterJob(); //Rcupration d'un travail d'impression.
        pageFormat = printJob.pageDialog(printJob.defaultPage());
      }

      ColoredTree toBePrinted;
      try {
        if(printWholeTree.isSelected()) {
          //Print the whole tree.
          //Clone the tree:
          toBePrinted = (ColoredTree)(tree.clone());
          printTree(toBePrinted, new File(chooseAFile.getText()));
        }
        if(printGroups.isSelected()) {
          //Print in separate files.
          toBePrinted=(ColoredTree)(GroupsManager.getGeneralTree(tree).clone());//Clonage obligatoire.
          toBePrinted.setName("main");
          printTree(toBePrinted);
          ColoredNode[] groups = (ColoredNode[])(GroupsManager.getGroups(tree).toArray(new ColoredNode[0]));
          String name;
          for(int i = 0; i < groups.length; i++) {
            name = groups[i].getName();//Sauvegarde du nom.
            groups[i].deleteName();//Effacement du nom.
            toBePrinted = new ColoredTree((ColoredNode)GroupsManager.getGeneralSubtree(groups[i]));
            groups[i].setName(name);//Rtablissement du nom.
            toBePrinted.setName(name);//Attribution du nom.
            printTree(toBePrinted);
          }
        }
        setVisible(false);
      } catch(NumberFormatException e) {
        error(Baobab.message.getString("printDialogEPSTooSmall"));
      }
    }

    if(source == cancel) {
      //on annule.
      setVisible(false);
    }
  }

  public void showUp() {
    this.tree = (ColoredTree)(mi.getCurrentTree().clone());
    printOnPrinter.setSelected(true);
    printLengths.setSelected(TreeTools.hasAtLeastOneLength(tree));
    printBootstraps.setSelected(TreeTools.hasAtLeastOneBootstrap(tree));
    actualize();
    setVisible(true);
  }

  public void error(String text) {//affichage d'un messsage d'erreur.
    JOptionPane.showMessageDialog(this, text, Baobab.message.getString("failure"), JOptionPane.ERROR_MESSAGE);
  }

  private boolean continu(String text) {
    boolean result = false;
    int choix = JOptionPane.showConfirmDialog(mi, text, Baobab.message.getString("warning"), JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
    if(choix == JOptionPane.OK_OPTION) result = true;
    return result;
  }

  private ColoredTree tree;
  private MainInterface mi;
  private JPanel panel1;
  private JButton ok, cancel;
  private JButton browseFile, browseDirectory;
  private JLabel chooseAFileLabel, chooseADirectoryLabel;
  private JTextField chooseAFile, chooseADirectory;
  private JFileChooser output_fc;
  private JFileChooser output_dc;
  private JTextField setNbMax;
  private JRadioButton printWholeTree, printGroups, printInPS, printOnPrinter;
  private JCheckBox printLengths, printBootstraps;
  private Container dialogContent;
  private TreePaginater tp;
  private PrinterJob printJob;
  private PageFormat pageFormat;
  private PostScriptFilter psFilter = new PostScriptFilter();
}
