// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

/**
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * @author Julien Dutheil
 */

package TreeEditor;

import bio.dendogram.xml.*;
import gnu.jtools.utils.attributes.Attribute;
import gnu.jtools.utils.attributes.AttributesMap;
import gnu.jtools.utils.attributes.BadAttributeNameException;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.*;

public class TreeAttributesEditor extends JSplitPane {
  // We no longer extend the AttributesEditor class and build our own design ;)

  protected JPanel bottomPanel;
  protected JTable table;
  protected AttributesTableModel atm;
  private static String[] columnNames = { "Attribute:", "Value:", "Reserved:", "Editable" };
  protected JPanel controlPanel;
  protected Action add, remove, copy, addInAllSubtree, removeInAllSubtree, copyInAllSubtree, renameInAllSubtree;
  protected JList treeAttributes;
  protected ColoredTreeDrawing treeDrawing;
  private ArrayList listeners;
  private static String yes = Baobab.message.getString("yes");
  private static String no  = Baobab.message.getString("no");

  public static class AttributesTableModel extends AbstractTableModel {

    public AttributesTableModel(AttributesMap am, Component owner) {
      this.owner = owner;
      Attribute[] atts = (Attribute[])am.entrySet().toArray(new Attribute[0]);
      Arrays.sort(atts, new Comparator() {
        public int compare(Object a, Object b) {
          return((Attribute)a).getName().compareTo(((Attribute)b).getName());
        }
      });
      data = new ArrayList(Arrays.asList(atts));
    }

    public int getRowCount() { return data.size(); }

    public int getColumnCount() { return 4; }

    public String getColumnName(int column) { return columnNames[column]; }

    public boolean isCellEditable(int row, int column) {
      switch(column) {
        case 0 : return getAttribute(row).isNameEditable();
        case 1 : return getAttribute(row).isValueEditable();
      }
      return false;
    }

    public Object getValueAt(int row, int column) {
      if(column == 0) return ((Attribute)data.get(row)).getKey();
      if(column == 1) return ((Attribute)data.get(row)).getValue();
      if(column == 2) return ((Attribute)data.get(row)).isNameEditable() ? "<html><b>" + no + "</b></html>" : "<html><b>" + yes + "</b></html>";
      if(column == 3) return ((Attribute)data.get(row)).isValueEditable() ? "<html><b>" + yes + "</b></html>" : "<html><b>" + no + "</b></html>";
      return null;
    }

    public void setValueAt(Object value, int row, int column) {
      if(!isCellEditable(row, column)) return;
      Attribute att = ((Attribute)data.get(row));
      if(column == 0) att.setName((String)value);
      if(column == 1) {
        try {
          Class type = att.getType();
          if (type.equals(Double.class)) att.setValue(new Double(value.toString()));
          else if (type.equals(Integer.class)) att.setValue(new Integer(value.
              toString()));
          else if (type.equals(Boolean.class)) att.setValue(new Boolean(value.
              toString()));
          else if (type.equals(String.class)) att.setValue(value.toString());
          else att.setValue(value);
        } catch(NumberFormatException nfe) {
          Baobab.error(owner, Baobab.message.getString("badNumber"));
        }
      }
      fireTableChanged(new TableModelEvent(this));
    }

    public void addRow(String name, String value) {
      Attribute att = new Attribute(name, value);
      data.add(att);
      fireTableChanged(new TableModelEvent(this));
    }

    public void addRow(String name, String value, boolean isNameEditable, boolean isValueEditable) {
      Attribute att = new Attribute(name, value, isNameEditable, isValueEditable);
      data.add(att);
      fireTableChanged(new TableModelEvent(this));
    }

    public void removeRow(int row) {
      if(row > -1 && row < this.getRowCount()) {
        data.remove(row);
        fireTableChanged(new TableModelEvent(this));
      }
    }

    public void duplicateRow(int row)
    throws NoSuchMethodException, InstantiationException, IllegalAccessException, InvocationTargetException {
      if(row > -1 && row < this.getRowCount()) {
        Attribute clone = (Attribute)getAttribute(row).deepClone();
        clone.setName("Copy of " + clone.getName());
        data.add(clone);
        fireTableChanged(new TableModelEvent(this));
      }
    }

    public Attribute getAttribute(int i) {
      return (Attribute)data.get(i);
    }

    public Attribute getAttribute(String name) {
      for(int i = 0; i < data.size(); i++) {
        Attribute att = (Attribute)data.get(i);
        if(att.getName().equals(name)) return att;
      }
      return null;
    }

    public AttributesMap getAttributes() {
      AttributesMap result = new AttributesMap();
      for(int i = 0; i < getRowCount(); i++) {
        result.put(this.getAttribute(i));
      }
      return result;
    }

    private ArrayList data;
    private Component owner;
  }

  public TreeAttributesEditor(ColoredTreeDrawing treeDrawing) {
    super(JSplitPane.VERTICAL_SPLIT);
    this.treeDrawing = treeDrawing;
    initActions();
    initGUI();
   }

  private void initActions() {

    add = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.Add")) {
      public void actionPerformed(ActionEvent e) {
        atm.addRow("New attribute", "value", true, true);
        table.updateUI();
      }
    };

    final TreeAttributesEditor attEd = this;
    remove = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.Remove")) {
      public void actionPerformed(ActionEvent e) {
        int[] rows = table.getSelectedRows();
        if(rows.length == 1) rows[0] = table.getSelectedRow();
        Arrays.sort(rows);//Very important!
        for(int i = rows.length - 1; i >= 0; i--) { //Must remove last first!
          if(atm.getAttribute(rows[i]).isValueEditable()) {
            atm.removeRow(rows[i]);
            table.updateUI();
          } else JOptionPane.showMessageDialog( attEd
              , "This attribute can't be removed: " + atm.getAttribute(rows[i]).getName()
              , "Error."
              , JOptionPane.ERROR_MESSAGE
              );
        }
      }
    };

    copy = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.Copy")) {
      public void actionPerformed(ActionEvent e) {
        int[] selectedRows = table.getSelectedRows();
        for(int i = 0; i < selectedRows.length; i++) {
          try {
            atm.duplicateRow(selectedRows[i]);
          } catch(Exception ex) {
            // Should never happen!
            Baobab.error(attEd, Baobab.message.getString("TreeAttributesEditor.error.CantDuplicate"));
          }
        }
        table.updateUI();
      }
    };

    addInAllSubtree = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.AddInAllSubtree")) {
      public void actionPerformed(ActionEvent e) {
        int[] selectedAttributes = table.getSelectedRows();
        for(int i = 0; i < selectedAttributes.length; i++) {
          Attribute a = atm.getAttribute(selectedAttributes[i]);
          if (a.isValueEditable()) {
            XTreeTools.setAttributeToAllSubtrees( ( (XNode)treeDrawing.getCurrentNode()), a);
            actualizeEditor();
          }
          else {
            showErrorMessage();
          }
        }
      }
    };

    removeInAllSubtree = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.RemoveInAllSubtree")) {
      public void actionPerformed(ActionEvent e) {
        int[] selectedAttributes = table.getSelectedRows();
        if(selectedAttributes.length == 1) selectedAttributes[0] = table.getSelectedRow(); //Prohibits a little Java bug...
        for(int i = 0; i < selectedAttributes.length; i++) {
          Attribute a = atm.getAttribute(selectedAttributes[i]);
          if(a.isValueEditable()) {
            XTreeTools.removeAttributeFromAllSubtrees((XNode)treeDrawing.getCurrentNode(), a);
            actualizeEditor();
          } else {
            showErrorMessage();
          }
        }
      }
    };

    copyInAllSubtree = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.CopyInAllSubtree")) {
      public void actionPerformed(ActionEvent e) {
        int[] selectedAttributes = table.getSelectedRows();
        for(int i = 0; i < selectedAttributes.length; i++) {
          Attribute a = atm.getAttribute(selectedAttributes[i]);
          String copyName = Baobab.request(attEd, Baobab.message.getString("TreeAttributesEditor.request.CopyName"), "Copy of " + a.getName());
          try {
            XTreeTools.copyAttributeForAllSubtrees( ( (XNode) treeDrawing.getCurrentNode()), a.getName(), copyName);
          }
          catch (BadAttributeNameException bane) {
            Baobab.error(attEd, Baobab.message.getString("TreeAttributesEditor.error.BadName") + " " + bane.getAttributeName());
          }
          catch (Exception ex) {
            // Should never happen!
            Baobab.error(attEd, Baobab.message.getString("TreeAttributesEditor.error.CantDuplicate"));
          }
          actualizeEditor();
        }
      }
    };

    renameInAllSubtree = new AbstractAction(Baobab.message.getString("TreeAttributesEditor.RenameInAllSubtree")) {
      public void actionPerformed(ActionEvent e) {
        int[] selectedAttributes = table.getSelectedRows();
        for(int i = 0; i < selectedAttributes.length; i++) {
          Attribute a = atm.getAttribute(selectedAttributes[i]);
          if(a.isNameEditable()) {
            String newName = Baobab.request(attEd, Baobab.message.getString("TreeAttributesEditor.request.NewName"), a.getName());
            try {
              XTreeTools.renameAttributeForAllSubtrees( ( (XNode) treeDrawing.getCurrentNode()), a.getName(), newName);
            } catch(BadAttributeNameException bane) {
              Baobab.error(attEd, Baobab.message.getString("TreeAttributesEditor.error.BadName") + " " + bane.getAttributeName());
            }
            actualizeEditor();
          } else {
            showErrorMessage();
          }
        }
      }
    };
  }

  private void initGUI() {

    atm = new AttributesTableModel(new AttributesMap(), this);
    table = new JTable(atm);
    table.addInputMethodListener(new InputMethodListener() {
      public void caretPositionChanged(InputMethodEvent ime) {}
      public void inputMethodTextChanged(InputMethodEvent ime) { actualizeAttributesLists(); }
    });

    initActions();

    JScrollPane scroll = new JScrollPane(table);
    scroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK), "Attributes:"));

    //controlPanel = new JPanel(new GridLayout(2, 2));
    //controlPanel.add(new JButton(add));
    //controlPanel.add(new JButton(remove));
    //controlPanel.add(new JButton(addToSubtree));
    //controlPanel.add(new JButton(removeFromSubtree));
    //controlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK), "Control:"));

    treeAttributes = new JList(new DefaultListModel());
    treeAttributes.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if(e.getClickCount() >= 2) {
          Object o = treeAttributes.getSelectedValue();
          if(o != null) {
            atm.addRow( (String) o, "", true, true);
            table.updateUI();
            actualizeAttributesLists();
          }
        }
      }
    });
    treeAttributes.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK), Baobab.message.getString("TreeAttributesEditor.AttributesInTree")));

    // Set layout:
    bottomPanel = new JPanel();
    JPanel bottomLPanel = new JPanel();
    JPanel bottomRPanel = new JPanel();
    bottomLPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK), Baobab.message.getString("TreeAttributesEditor.ThisNode")));
    bottomRPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.BLACK), Baobab.message.getString("TreeAttributesEditor.ThisSubtree")));
    bottomLPanel.setLayout(new GridBagLayout());
    bottomRPanel.setLayout(new GridBagLayout());
    bottomPanel.setLayout(new GridBagLayout());

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = gbc.BOTH;
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 50;
    gbc.weighty = 0;
    bottomPanel.add(bottomLPanel, gbc);
    gbc.gridx = 1;
    bottomPanel.add(bottomRPanel, gbc);

    gbc.weightx = 100;
    gbc.weighty = 0;
    gbc.gridx = 0;
    gbc.gridy = 0;
    bottomLPanel.add(new JButton(add), gbc);

    gbc.gridy = 1;
    bottomLPanel.add(new JButton(remove), gbc);

    gbc.gridy = 2;
    bottomLPanel.add(new JButton(copy), gbc);

    gbc.gridy = 0;
    bottomRPanel.add(new JButton(addInAllSubtree), gbc);

    gbc.gridy = 1;
    bottomRPanel.add(new JButton(removeInAllSubtree), gbc);

    gbc.gridy = 2;
    bottomRPanel.add(new JButton(copyInAllSubtree), gbc);

    gbc.gridy = 3;
    bottomRPanel.add(new JButton(renameInAllSubtree), gbc);

    gbc.weightx = 1;
    gbc.weighty = 1;

    gbc.gridwidth = 2;
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.weightx = 0;
    gbc.weighty = 100;
    bottomPanel.add(new JScrollPane(treeAttributes), gbc);

    this.setTopComponent(scroll);
    this.setBottomComponent(bottomPanel);
    this.setDividerLocation(200);

    listeners = new ArrayList();
  }

  public void addTableModelListener(TableModelListener listener) {
    listeners.add(listener);
    atm.addTableModelListener(listener);
  }

  public String getAttributeValue(String attributeName) {
    for(int i = 0; i < atm.getRowCount(); i++) if(((String)atm.getValueAt(i, 0)).equals(attributeName)) return (String)atm.getValueAt(i, 1);
    return null;
  }

  public String getAttributeName(int index) {
    return (String)atm.getValueAt(index, 0);
  }

  public String getAttributeValue(int index) {
    return (String)atm.getValueAt(index, 1);
  }

  public AttributesTableModel getAttributesTableModel() { return atm; }

  public void setAttributesTableModel(AttributesTableModel atm) {
    this.atm = atm;
    for(int i = 0; i < listeners.size(); i++) atm.addTableModelListener((TableModelListener)listeners.get(i));
    //System.out.println("Attributes table model changed");
    table.setModel(atm);
  }

  public void actualizeAttributesLists() {
    ((DefaultListModel)treeAttributes.getModel()).removeAllElements();
    Set tags = XTreeTools.getAvailableTags((XTree)treeDrawing.getTree(), false);
    Iterator i = tags.iterator();
    while(i.hasNext()) {
      String s = (String)i.next();
      Attribute att = atm.getAttribute(s);
      if(att == null) ((DefaultListModel)treeAttributes.getModel()).addElement(s);
    }
  }

  public void actualizeEditor() {
    setAttributesTableModel(new AttributesTableModel(((XNode)treeDrawing.getCurrentNode()).getAttributes(), this));
    actualizeAttributesLists();
  }

  private void showErrorMessage() {
    JOptionPane.showMessageDialog( this
                                 , "This attribute is not editable."
                                 , "Error!"
                                 , JOptionPane.ERROR_MESSAGE
                                 );
  }
}
