// [English version follows the french one]
//
// Copyright ou  ou Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
// Ce logiciel est un programme informatique servant  diter des arbres
// phylogntiques.
//
// Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
// respectant les principes de diffusion des logiciels libres. Vous pouvez
// utiliser, modifier et/ou redistribuer ce programme sous les conditions
// de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
// sur le site "http://www.cecill.info".
//
// En contrepartie de l'accessibilit au code source et des droits de copie,
// de modification et de redistribution accords par cette licence, il n'est
// offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
// seule une responsabilit restreinte pse sur l'auteur du programme,  le
// titulaire des droits patrimoniaux et les concdants successifs.
//
// A cet gard  l'attention de l'utilisateur est attire sur les risques
// associs au chargement,   l'utilisation,   la modification et/ou au
// dveloppement et  la reproduction du logiciel par l'utilisateur tant
// donn sa spcificit de logiciel libre, qui peut le rendre complexe 
// manipuler et qui le rserve donc  des dveloppeurs et des professionnels
// avertis possdant  des  connaissances  informatiques approfondies.  Les
// utilisateurs sont donc invits  charger  et  tester  l'adquation  du
// logiciel  leurs besoins dans des conditions permettant d'assurer la
// scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
//  l'utiliser et l'exploiter dans les mmes conditions de scurit.
//
// Le fait que vous puissiez accder  cet en-tte signifie que vous avez
// pris connaissance de la licence CeCILL, et que vous en avez accept les
// termes.
//
// -------------------------------------------------------------------------
//
//Copyright or  or Copr. Julien Dutheil, UMR 5171 "Gnome, populations,
// interactions, adaptation" CNRS, Universit Montpellier II, IFREMER
// (Septembre 2004)
//
// Julien.Dutheil@univ-montp2.fr
//
//
// This software is a computer program whose purpose is to edit
// phylogenetic trees.
//
// This software is governed by the CeCILL  license under French law and
// abiding by the rules of distribution of free software.  You can  use,
// modify and/ or redistribute the software under the terms of the CeCILL
// license as circulated by CEA, CNRS and INRIA at the following URL
// "http://www.cecill.info".
//
// As a counterpart to the access to the source code and  rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty  and the software's author,  the holder of the
// economic rights,  and the successive licensors  have only  limited
// liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading,  using,  modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean  that it is complicated to manipulate,  and  that  also
// therefore means  that it is reserved for developers  and  experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and,  more generally, to use and operate it in the
// same conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.
//
// -------------------------------------------------------------------------

package TreeEditor;

/**
 * <p>Titre : Tree Editor</p>
 * <p>Description : Baobab core classes.</p>
 * <p>Copyright : Copyright (c) Julien Dutheil</p>
 * <p>Socit : </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import gnu.jtools.stats.array.ArrayOperators;
import gnu.jtools.utils.string.TextTools;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import bio.dendogram.*;

public class TreeNameInputDialog extends JDialog {

  public TreeNameInputDialog(final MainInterface mi) {
    super(mi, mi.getTitle(), true);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { name = null; }
    });
    this.mi = mi;
    Box box = Box.createVerticalBox();
    getContentPane().add(box);
    setName = new JTextField(Baobab.message.getString("loadDialogDefaultTreeName"));
    box.add(setName);

    JTextArea txt = new JTextArea(Baobab.message.getString("loadDialogAskTreeName"));
              txt.setEditable(false);
              txt.setLineWrap(true);
              txt.setWrapStyleWord(true);
              txt.setForeground(Color.blue);
              txt.setBorder(BorderFactory.createEtchedBorder());
    box.add(txt);

    list = new JTextArea();
    list.setEditable(false);
    list.setForeground(Color.green);

    JScrollPane lift = new JScrollPane(list);
                lift.setPreferredSize(new Dimension(300, 200));
                lift.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Baobab.message.getString("loadDialogLeaves")));
    box.add(lift);

    ok = new JButton(Baobab.message.getString("ok"));
    ok.setMnemonic(Baobab.message.getString("okMnemo").toCharArray()[0]);
    ok.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String n = setName.getText();
        if(n == null
           || TextTools.isVoid(n)
           || ArrayOperators.indexOf(mi.getList().getSavedNames(), n) > -1) {//nom dj utilis
          error(Baobab.message.getString("loadDialogBadTreeName"));
        } else {
          name = n;
          setVisible(false);
        }
      }
    });
    getContentPane().add(ok, BorderLayout.SOUTH);
    pack();
    Toolkit tk = Toolkit.getDefaultToolkit();
    setBounds((tk.getScreenSize().width-getWidth())/2, (tk.getScreenSize().height-getHeight())/2, getWidth(), getHeight());
  }

  public String getName(Tree tree) {
    if(tree.getName() != null) setName.setText(tree.getName());

    java.util.List leaves = tree.listOfLeavesNames();
    StringBuffer sb = new StringBuffer();
    for(int i = 0; i < leaves.size() - 1; i++) {
      sb.append((String)leaves.get(i)+"\n");
    }
    if(leaves.size() > 0) sb.append( (String)leaves.get(leaves.size() - 1) );
    list.setText(sb.toString());
    setVisible(true);
    return name;
  }

  public void error(String text) {//affichage d'un messsage d'erreur.
    JOptionPane.showMessageDialog(this, text, Baobab.message.getString("failure"), JOptionPane.ERROR_MESSAGE);
  }

  private MainInterface mi;
  private JTextField setName;
  private JTextArea list;
  private JButton ok;
  private String name;
}
