package bio.sequences.io;

/**
 * <p>Title: Bio</p>
 * <p>Description: Tools for bioinformatics</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Julien Dutheil
 * @version 1.0
 */

import bio.sequences.*;
import gnu.jtools.utils.file.FileTools;
import gnu.jtools.utils.string.TextTools;
import java.io.*;
import javax.swing.filechooser.*;

/**
 * @brief The Fasta file format.
 *
 * Read and write from/to Fasta files.
 */
public class Fasta extends AbstractIOSequence {

  /**
   * @brief Build a new Fasta object.
   *
   * @param charsByLine Number of character per line when writing files.
   */
  public Fasta(int charsByLine) {
    super(charsByLine);
  }

  public Fasta() {
    super();
  }

  /**
   * @name The ISEquence interface.
   *
   * @{
   */
  public void read(File file, VectorSequenceContainer vsc) throws Exception {
    BufferedReader br = new BufferedReader(new FileReader(file));
    String temp = null, name = null, sequence = "";  // Initialization

    // Main loop : for all file lines
    temp = br.readLine();
    while(temp != null) {
      if(!TextTools.isVoid(temp)) {
        // If first character is >
        if (temp.charAt(0) == '>') {
          // If a name and a sequence were founded
          if (name != null && !sequence.equals("")) {
            // New sequence creation, and addition in existing VectorSequenceContainer
            Sequence seq = new Sequence(name, sequence, vsc.getAlphabet());
            vsc.addSequence(seq, true);
          }
          // Sequence name isolation
          name = TextTools.removeFirstBlanks(temp.substring(1));// Remove character '>'
          sequence = "";
        } else sequence += TextTools.removeBlanks(temp);  // Sequence isolation
      }
      temp = br.readLine();
    }

    // Addition of the last sequence in file
    if(name != null && sequence != "") {
      Sequence seq = new Sequence(name, sequence, vsc.getAlphabet());
      vsc.addSequence(seq, true);
    }

    br.close();
  }
  /** @} */

  /**
   * @name The OSequence interface.
   *
   * @{
   */
  public void write(File file, SequenceContainer sc, boolean overwrite) throws Exception {
    // Open file in specified mode
    PrintWriter pw = new PrintWriter(new FileWriter(file, !overwrite));

    //StringBuffer seq, temp = new StringBuffer("");  // Initialization

    // Main loop : for all sequences in vector container
    try {
      String[] names = sc.getSequencesNames();
      for (int i = 0; i < names.length; i++) {
        pw.println(">" + names[i]);

        // Sequence cutting to specified characters number per line
        String[] seq = TextTools.split(sc.toString(names[i]), _charsByLine);
        for(int j = 0; j < seq.length; j++) {
          pw.println(seq[j]);
        }
      }
    } catch(Exception e) { e.printStackTrace(); }//Never thrown?
    pw.close();
  }
  /** @} */

  /**
   * @name The IOSequence interface.
   *
   * @{
   */
  public String getFormatName() {
    return "FASTA file";
  }

  public String getFormatDescription() {
    return "Sequence name (preceded by >) in one line, sequence content, no comments";
  }
  /** @} */

  public static javax.swing.filechooser.FileFilter FASTA_FILTER = new javax.swing.filechooser.FileFilter() {

      public boolean accept(java.io.File file) {
        if(file.isDirectory()) return true;
        String extension = FileTools.getExtension(file);
        if(extension != null) {
          if(extension.equals("fasta") ||
             extension.equals("fas")) return true;
          else return false;
        }
        return false;
      }

      // The description of this filter
      public String getDescription() {
        return "Fasta files (*.fasta, *.fas)";
      }
    };

}
